@@ -1,6 +1,6 @@
 static const char	uget_license[] =
 {
-" Copyright (C) 2005-2012 by C.H. Huang"								"\n"
+" Copyright (C) 2005-2013 by C.H. Huang"								"\n"
 " plushuang.tw@gmail.com"												"\n"
 																		"\n"
 "This library is free software; you can redistribute it and/or"			"\n"
@@ -38,11 +38,6 @@ static const char	uget_license[] =
 #endif
 
 #include <gdk/gdkkeysyms.h>    // for GDK_KEY_xxx...
-// for GTK+ 2.18
-#ifndef GDK_KEY_Menu
-#define GDK_KEY_Menu		GDK_Menu
-#define GDK_KEY_Delete		GDK_Delete
-#endif
 
 // uglib
 #include <UgHtml.h>
@@ -55,20 +50,21 @@ static const char	uget_license[] =
 
 #include <glib/gi18n.h>
 
-#define	UGET_URL_WEBSITE	"http://uget.visuex.com/"
+#define	UGET_URL_WEBSITE	"http://uGet.visuex.com/"
 
 // static data
 static const gchar*	uget_authors[] = { "C.H. Huang  (\xE9\xBB\x83\xE6\xAD\xA3\xE9\x9B\x84)", NULL };
 static const gchar*	uget_artists[] =
 {
 	"Logo designer: Michael Tunnell (visuex.com)",
+	"Website by: Michael Tunnell (visuex.com)",
 	"Former Logo designer: saf1 (linuxac.org)",
 	"Former Logo improver: Skeleton_Eel (linuxac.org)",
 	NULL
 };
 static const gchar*	uget_version   = UG_APP_GTK_VERSION;
 static const gchar*	uget_comments  = N_("Download Manager");
-static const gchar*	uget_copyright = "Copyright (C) 2005-2012 C.H. Huang";
+static const gchar*	uget_copyright = "Copyright (C) 2005-2013 C.H. Huang";
 static const gchar*	translator_credits = N_("translator-credits");
 
 // static functions
@@ -199,12 +195,40 @@ static void	on_config_category (GtkWidget* widget, UgAppGtk* app)
 // ----------------------------------------------------------------------------
 // Download
 //
+static void	ug_app_setup_download_dialog (UgAppGtk* app, UgDownloadDialog* ddialog)
+{
+	UgDataCommon*	common;
+	GtkTreePath*	path;
+	GtkTreeModel*	model;
+
+	ug_download_form_set_folder_list (&ddialog->download,
+			app->setting.folder_list);
+	ug_download_dialog_set_category (ddialog, &app->cwidget);
+
+	if (app->setting.ui.apply_recently && app->last.download) {
+		model = gtk_tree_view_get_model (ddialog->category_view);
+		if (app->last.category_index < gtk_tree_model_iter_n_children (model, NULL)) {
+			path = gtk_tree_path_new_from_indices (app->last.category_index, -1);
+			gtk_tree_view_set_cursor (ddialog->category_view, path, NULL, FALSE);
+			gtk_tree_path_free (path);
+		}
+
+		common = ug_dataset_realloc (app->last.download, UG_DATA_COMMON_I, 0);
+		if (common && common->file) {
+			g_free (common->file);
+			common->file = NULL;
+		}
+		ug_download_dialog_set (ddialog, app->last.download);
+	}
+}
+
 static void	on_create_download_response (GtkDialog *dialog, gint response_id, UgDownloadDialog* ddialog)
 {
 	UgCategory*		category;
 	UgAppGtk*		app;
 	GList*			list;
 	GList*			link;
+	GtkTreePath*	path;
 
 	if (response_id == GTK_RESPONSE_OK) {
 		app = ddialog->user.app;
@@ -215,6 +239,20 @@ static void	on_create_download_response (GtkDialog *dialog, gint response_id, Ug
 			list = ug_download_dialog_get_downloads (ddialog);
 			for (link = list;  link;  link = link->next)
 				ug_category_add (category, link->data);
+			// get last download settings
+			if (list->data) {
+				if (app->last.download)
+					ug_dataset_unref (app->last.download);
+				ug_dataset_ref (list->data);
+				app->last.download = list->data;
+			}
+			// get last category index
+			gtk_tree_view_get_cursor (ddialog->category_view, &path, NULL);
+			if (path) {
+				app->last.category_index = *gtk_tree_path_get_indices (path);
+				gtk_tree_path_free (path);
+			}
+			// free unused
 			g_list_foreach (list, (GFunc) ug_dataset_unref, NULL);
 			g_list_free (list);
 			gtk_widget_queue_draw ((GtkWidget*) app->cwidget.self);
@@ -234,14 +272,16 @@ static void	on_create_download (GtkWidget* widget, UgAppGtk* app)
 	g_free (title);
 	if (gtk_widget_get_visible ((GtkWidget*) app->window.self) == FALSE)
 		gtk_window_set_transient_for ((GtkWindow*) ddialog->self, NULL);
-	ug_download_form_set_folder_list (&ddialog->download,
-			app->setting.folder_list);
-	ug_download_dialog_set_category (ddialog, &app->cwidget);
+	// setup download dialog
+	ug_app_setup_download_dialog (app, ddialog);
+	// use first URI from clipboard to set URL entry
 	list = ug_clipboard_get_uris (&app->clipboard);
 	if (list) {
+		ddialog->download.changed.url = TRUE;
 		gtk_entry_set_text ((GtkEntry*) ddialog->download.url_entry, list->data);
 		g_list_foreach (list, (GFunc) g_free, NULL);
 		g_list_free (list);
+		ug_download_form_complete_entry (&ddialog->download);
 	}
 	// connect signal and set data in download dialog
 	ddialog->user.app = app;
@@ -250,18 +290,17 @@ static void	on_create_download (GtkWidget* widget, UgAppGtk* app)
 	gtk_widget_show ((GtkWidget*) ddialog->self);
 }
 
-static void	on_create_batch (GtkWidget* widget, UgAppGtk* app)
+static void	on_create_sequence (GtkWidget* widget, UgAppGtk* app)
 {
 	UgDownloadDialog*	ddialog;
 	gchar*				title;
 
-	title = g_strconcat (UG_APP_GTK_NAME " - ", _("New Batch Download"), NULL);
+	title = g_strconcat (UG_APP_GTK_NAME " - ", _("URL Sequence batch"), NULL);
 	ddialog = ug_download_dialog_new (title, app->window.self);
 	g_free (title);
-	ug_download_form_set_folder_list (&ddialog->download,
-			app->setting.folder_list);
-	ug_download_dialog_set_category (ddialog, &app->cwidget);
 	ug_download_dialog_use_batch (ddialog);
+	// setup download dialog
+	ug_app_setup_download_dialog (app, ddialog);
 	// connect signal and set data in download dialog
 	ddialog->user.app = app;
 	g_signal_connect (ddialog->self, "response",
@@ -283,17 +322,19 @@ static void	on_create_from_clipboard (GtkWidget* widget, UgAppGtk* app)
 		return;
 	}
 
-	title = g_strconcat (UG_APP_GTK_NAME " - ", _("New from Clipboard"), NULL);
+	title = g_strconcat (UG_APP_GTK_NAME " - ", _("Clipboard batch"), NULL);
 	ddialog = ug_download_dialog_new (title, app->window.self);
 	g_free (title);
 	if (gtk_widget_get_visible ((GtkWidget*) app->window.self) == FALSE)
 		gtk_window_set_transient_for ((GtkWindow*) ddialog->self, NULL);
-	ug_download_dialog_set_category (ddialog, &app->cwidget);
 	ug_download_dialog_use_selector (ddialog);
+	// selector
 	ug_selector_hide_href (&ddialog->selector);
 	page = ug_selector_add_page (&ddialog->selector, _("Clipboard"));
 	ug_selector_page_add_uris (page, list);
 	g_list_free (list);
+	// setup download dialog
+	ug_app_setup_download_dialog (app, ddialog);
 	// connect signal and set data in download dialog
 	ddialog->user.app = app;
 	g_signal_connect (ddialog->self, "response",
@@ -411,17 +452,24 @@ static void	on_delete_download (GtkWidget* widget, UgAppGtk* app)
 	GList*				link;
 	// check shift key status
 	GdkWindow*			gdk_win;
+	GdkDevice*			dev_pointer;
 	GdkModifierType		mask;
 
 	dwidget = app->cwidget.current.widget;
 	// check shift key status
 	gdk_win = gtk_widget_get_parent_window ((GtkWidget*) dwidget->view);
-	gdk_window_get_pointer (gdk_win, NULL, NULL, &mask);
+	dev_pointer = gdk_device_manager_get_client_pointer (
+			gdk_display_get_device_manager (gdk_window_get_display (gdk_win)));
+	gdk_window_get_device_position (gdk_win, dev_pointer, NULL, NULL, &mask);
+
+	// clear summary
+	ug_summary_show (&app->summary, NULL);
+	// set action status "stop by user" and "deleted"
+	app->action.stop = TRUE;
+	app->action.deleted = TRUE;
 
 	list = ug_download_widget_get_selected (dwidget);
 	for (link = list;  link;  link = link->next) {
-		// set status "stop by user"
-		app->user_action = TRUE;
 		// stop task
 		ug_running_remove (&app->running, link->data);
 		// delete data or move it to recycled
@@ -434,6 +482,8 @@ static void	on_delete_download (GtkWidget* widget, UgAppGtk* app)
 		}
 	}
 	g_list_free (list);
+
+	app->action.deleted = FALSE;
 	// update
 	gtk_widget_queue_draw ((GtkWidget*) app->cwidget.self);
 	ug_summary_show (&app->summary, ug_download_widget_get_cursor (dwidget));
@@ -451,10 +501,14 @@ static void	on_delete_download_file_response (GtkWidget* widget, gint response_i
 		return;
 
 	dwidget = app->cwidget.current.widget;
+	// clear summary
+	ug_summary_show (&app->summary, NULL);
+	// set action status "stop by user" and "deleted"
+	app->action.stop = TRUE;
+	app->action.deleted = TRUE;
+
 	list = ug_download_widget_get_selected (dwidget);
 	for (link = list;  link;  link = link->next) {
-		// set status "stop by user"
-		app->user_action = TRUE;
 		// stop task
 		ug_running_remove (&app->running, link->data);
 		// delete file
@@ -470,6 +524,8 @@ static void	on_delete_download_file_response (GtkWidget* widget, gint response_i
 		ug_category_gtk_remove (app->cwidget.current.category, link->data);
 	}
 	g_list_free (list);
+
+	app->action.deleted = FALSE;
 	// update
 	gtk_widget_queue_draw ((GtkWidget*) app->cwidget.self);
 	ug_summary_show (&app->summary, ug_download_widget_get_cursor (dwidget));
@@ -685,13 +741,14 @@ static void	on_set_download_to_pause (GtkWidget* widget, UgAppGtk* app)
 	GList*				list;
 	GList*				link;
 
+	// set action status "stop by user"
+	app->action.stop = TRUE;
+
 	dwidget = app->cwidget.current.widget;
 	list = ug_download_widget_get_selected (dwidget);
 	for (link = list;  link;  link = link->next) {
 		relation = UG_DATASET_RELATION ((UgDataset*) link->data);
 		relation->hints |=  UG_HINT_PAUSED;
-		// set status "stop by user"
-		app->user_action = TRUE;
 		// stop task
 		ug_running_remove (&app->running, link->data);
 	}
@@ -997,8 +1054,8 @@ static void	on_offline_mode (GtkWidget* widget, UgAppGtk* app)
 
 	// into offline mode
 	if (app->setting.offline_mode == TRUE) {
-		// set status "stop by user"
-		app->user_action = TRUE;
+		// set action status "stop by user"
+		app->action.stop = TRUE;
 		// stop all active tasks
 		ug_running_clear (&app->running);
 		// refresh
@@ -1043,6 +1100,7 @@ static void	on_config_settings_response (GtkDialog *dialog, gint response, UgSet
 		// aria2
 		ug_app_decide_bt_meta_sensitive (app);
 		ug_app_aria2_setup (app);
+		app->aria2.remote_updated = FALSE;
 	}
 	ug_setting_dialog_free (sdialog);
 	// refresh
@@ -1126,7 +1184,7 @@ void	on_change_visible_widget (GtkWidget* widget, UgAppGtk* app)
 			gtk_widget_hide (app->cwidget.self);
 		return;
 	}
-	// Category
+	// Summary
 	if (widget == app->menubar.view.summary) {
 		setting->summary = visible;
 		if (visible)
@@ -1253,13 +1311,21 @@ void	on_change_visible_column (GtkWidget* widget, UgAppGtk* app)
 //
 void	on_about (GtkWidget* widget, UgAppGtk* app)
 {
+	GtkImage* img;
+	char* path;
+
+	path = g_build_filename (ug_get_data_dir (), "pixmaps", "uget", "logo.png", NULL);
+	img = (GtkImage*) gtk_image_new_from_file (path);
+	g_free (path);
+
 //	if (gtk_widget_get_visible ((GtkWidget*) app->window.self) == FALSE) {
 //		gtk_window_deiconify (app->window.self);
 //		gtk_widget_show ((GtkWidget*) app->window.self);
 //	}
 
 	gtk_show_about_dialog (app->window.self,
-			"logo-icon-name", UG_APP_GTK_ICON_NAME,
+//			"logo-icon-name", UG_APP_GTK_APP_ICON_NAME,
+			"logo", gtk_image_get_pixbuf (img),
 			"program-name", UG_APP_GTK_NAME,
 			"version", uget_version,
 			"comments", gettext (uget_comments),
@@ -1274,12 +1340,13 @@ void	on_about (GtkWidget* widget, UgAppGtk* app)
 			"artists", uget_artists,
 			"translator-credits", gettext (translator_credits),
 			NULL);
+
+	g_object_unref (img);
 }
 
 // ----------------------------------------------------------------------------
 // UgTrayIcon
 //
-#ifndef HAVE_APP_INDICATOR
 static void	on_trayicon_activate (GtkStatusIcon* status_icon, UgAppGtk* app)
 {
 	if (gtk_widget_get_visible ((GtkWidget*) app->window.self) == TRUE) {
@@ -1289,19 +1356,23 @@ static void	on_trayicon_activate (GtkStatusIcon* status_icon, UgAppGtk* app)
 		gtk_window_get_size (app->window.self,
 				&app->setting.window.width, &app->setting.window.height);
 		// hide window
+#ifdef _WIN32
 		gtk_window_iconify (app->window.self);
+#endif
 		gtk_widget_hide ((GtkWidget*) app->window.self);
 	}
 	else {
+#ifdef _WIN32
 		gtk_widget_show ((GtkWidget*) app->window.self);
 		gtk_window_deiconify (app->window.self);
+#endif
 		gtk_window_present (app->window.self);
 		ug_app_trayicon_decide_visible (app);
 	}
 	// clear error status
 	if (app->trayicon.error_occurred) {
 		app->trayicon.error_occurred = FALSE;
-		gtk_status_icon_set_from_icon_name (status_icon, UG_APP_GTK_ICON_NAME);
+		gtk_status_icon_set_from_icon_name (status_icon, UG_APP_GTK_TRAY_ICON_NAME);
 	}
 }
 
@@ -1321,7 +1392,6 @@ static void	on_trayicon_popup_menu (GtkStatusIcon* status_icon, guint button, gu
 			button, activate_time);
 #endif
 }
-#endif	// HAVE_APP_INDICATOR
 
 static void	on_trayicon_show_window (GtkWidget* widget, UgAppGtk* app)
 {
@@ -1385,7 +1455,7 @@ static void menu_position_func (GtkMenu*	menu,
 
 	widget = user_data;
 	gdk_window_get_origin (gtk_widget_get_window (widget), x, y);
-	gtk_widget_size_request (GTK_WIDGET (menu), &menu_requisition);
+	gtk_widget_get_preferred_size (GTK_WIDGET (menu), &menu_requisition, NULL);
 
 	gtk_widget_get_allocation (widget, &allocation);
 	if (gtk_widget_get_direction (widget) == GTK_TEXT_DIR_RTL)
@@ -1462,6 +1532,38 @@ static gboolean	on_download_key_press_event  (GtkWidget* widget, GdkEventKey* ev
 		on_delete_download (widget, app);
 		return TRUE;
 	}
+/*
+	// check shift key status
+	GdkWindow*		gdk_win;
+	GdkDevice*		dev_pointer;
+	GdkModifierType	mask;
+
+	// check shift key status
+	gdk_win = gtk_widget_get_parent_window ((GtkWidget*) app->cwidget.current.widget->view);
+	dev_pointer = gdk_device_manager_get_client_pointer (
+			gdk_display_get_device_manager (gdk_window_get_display (gdk_win)));
+	gdk_window_get_device_position (gdk_win, dev_pointer, NULL, NULL, &mask);
+
+	if (app->cwidget.current.category) {
+		switch (event->keyval) {
+		case GDK_KEY_Delete:
+		case GDK_KEY_KP_Delete:
+			if (mask & GDK_SHIFT_MASK)
+				on_delete_download_file (widget, app);
+			else
+				on_delete_download (widget, app);
+			return TRUE;
+
+		case GDK_KEY_Return:
+		case GDK_KEY_KP_Enter:
+			if (mask & GDK_SHIFT_MASK)
+				on_open_download_folder (widget, app);
+			else
+				on_open_download_file (widget, app);
+			return TRUE;
+		}
+	}
+*/
 	return FALSE;
 }
 
@@ -1481,9 +1583,18 @@ static void	on_download_cursor_changed (GtkTreeView* view, UgAppGtk* app)
 	UgDownloadWidget*	dwidget;
 	UgDataset*			dataset;
 
-	dwidget = app->cwidget.current.widget;
-	dataset = ug_download_widget_get_cursor (dwidget);
-	ug_summary_show (&app->summary, dataset);
+//	Add "action.deleted" to avoid crash in GTK+ 3.4. This maybe GTK+ bug.
+//	1. user delete an iter in GtkTreeModel.
+//	2. emit "cursor-changed" signal because cursor changed after deleting.
+//	3. call gtk_tree_view_get_cursor(view, &path)
+//	4. call gtk_tree_model_get_iter(model, &iter, path) and result TRUE.
+//	5. call gtk_tree_model_get(model, &iter, ...) may result invalid iter
+//	   and crash...
+	if (app->action.deleted == FALSE) {
+		dwidget = app->cwidget.current.widget;
+		dataset = ug_download_widget_get_cursor (dwidget);
+		ug_summary_show (&app->summary, dataset);
+	}
 }
 
 // UgCategoryWidget.view and primary_view "cursor-changed"
@@ -1618,12 +1729,10 @@ static gboolean	tray_menu_leave_enter (GtkWidget* menu, GdkEventCrossing* event,
 // UgTrayIcon
 static void ug_trayicon_init_callback (struct UgTrayIcon* icon, UgAppGtk* app)
 {
-#ifndef HAVE_APP_INDICATOR
 	g_signal_connect (icon->self, "activate",
 			G_CALLBACK (on_trayicon_activate), app);
 	g_signal_connect (icon->self, "popup-menu",
 			G_CALLBACK (on_trayicon_popup_menu), app);
-#endif
 
 #ifdef _WIN32
 	g_signal_connect (icon->menu.self, "leave-notify-event",
@@ -1662,8 +1771,8 @@ static void ug_toolbar_init_callback (struct UgToolbar* toolbar, UgAppGtk* app)
 			G_CALLBACK (on_create_download), app);
 	g_signal_connect (toolbar->create_category, "activate",
 			G_CALLBACK (on_create_category), app);
-	g_signal_connect (toolbar->create_batch, "activate",
-			G_CALLBACK (on_create_batch), app);
+	g_signal_connect (toolbar->create_sequence, "activate",
+			G_CALLBACK (on_create_sequence), app);
 	g_signal_connect (toolbar->create_clipboard, "activate",
 			G_CALLBACK (on_create_from_clipboard), app);
 	g_signal_connect (toolbar->create_torrent, "activate",
@@ -1701,22 +1810,22 @@ static void ug_menubar_init_callback (struct UgMenubar* menubar, UgAppGtk* app)
 			G_CALLBACK (on_create_download), app);
 	g_signal_connect (menubar->file.create.category, "activate",
 			G_CALLBACK (on_create_category), app);
-	g_signal_connect (menubar->file.create.batch, "activate",
-			G_CALLBACK (on_create_batch), app);
-	g_signal_connect (menubar->file.create.from_clipboard, "activate",
-			G_CALLBACK (on_create_from_clipboard), app);
 	g_signal_connect (menubar->file.create.torrent, "activate",
 			G_CALLBACK (on_create_torrent), app);
 	g_signal_connect (menubar->file.create.metalink, "activate",
 			G_CALLBACK (on_create_metalink), app);
+	g_signal_connect (menubar->file.batch.clipboard, "activate",
+			G_CALLBACK (on_create_from_clipboard), app);
+	g_signal_connect (menubar->file.batch.sequence, "activate",
+			G_CALLBACK (on_create_sequence), app);
+	g_signal_connect (menubar->file.batch.text_import, "activate",
+			G_CALLBACK (on_import_text_file), app);
+	g_signal_connect (menubar->file.batch.html_import, "activate",
+			G_CALLBACK (on_import_html_file), app);
+	g_signal_connect (menubar->file.batch.text_export, "activate",
+			G_CALLBACK (on_export_text_file), app);
 	g_signal_connect_swapped (menubar->file.save, "activate",
 			G_CALLBACK (ug_app_save), app);
-	g_signal_connect (menubar->file.import_html, "activate",
-			G_CALLBACK (on_import_html_file), app);
-	g_signal_connect (menubar->file.import_text, "activate",
-			G_CALLBACK (on_import_text_file), app);
-	g_signal_connect (menubar->file.export_text, "activate",
-			G_CALLBACK (on_export_text_file), app);
 	g_signal_connect (menubar->file.offline_mode, "toggled",
 			G_CALLBACK (on_offline_mode), app);
 	g_signal_connect_swapped (menubar->file.quit, "activate",
