@@ -1,6 +1,6 @@
 /*
  *
- *   Copyright (C) 2005-2012 by C.H. Huang
+ *   Copyright (C) 2005-2013 by C.H. Huang
  *   plushuang.tw@gmail.com
  *
  *  This library is free software; you can redistribute it and/or
@@ -35,10 +35,6 @@
  */
 
 #include <gdk/gdkkeysyms.h>    // for GDK_KEY_key...
-// for GTK+ 2.18
-#ifndef GDK_KEY_n
-#define GDK_KEY_n			GDK_n
-#endif
 
 #ifdef	HAVE_CONFIG_H
 #include <config.h>
@@ -68,7 +64,12 @@ void	ug_app_init_gui (UgAppGtk* app)
 #endif	// _WIN32
 
 	// Registers a new accelerator "Ctrl+N" with the global accelerator map.
-	gtk_accel_map_add_entry (UG_APP_GTK_ACCEL_PATH_CTRL_N, GDK_KEY_n, GDK_CONTROL_MASK);
+	gtk_accel_map_add_entry (UG_APP_GTK_ACCEL_PATH_CTRL_N,   GDK_KEY_n,      GDK_CONTROL_MASK);
+	gtk_accel_map_add_entry (UG_APP_GTK_ACCEL_PATH_DELETE,   GDK_KEY_Delete, 0);
+//	gtk_accel_map_add_entry (UG_APP_GTK_ACCEL_PATH_DELETE_F, GDK_KEY_Delete, GDK_SHIFT_MASK);
+	gtk_accel_map_add_entry (UG_APP_GTK_ACCEL_PATH_DELETE_F, GDK_KEY_Delete, GDK_CONTROL_MASK);
+	gtk_accel_map_add_entry (UG_APP_GTK_ACCEL_PATH_OPEN,     GDK_KEY_Return, 0);
+	gtk_accel_map_add_entry (UG_APP_GTK_ACCEL_PATH_OPEN_F,   GDK_KEY_Return, GDK_SHIFT_MASK);
 	// accelerators
 	app->accel_group = gtk_accel_group_new ();
 	// tray icon
@@ -102,8 +103,8 @@ static void ug_trayicon_init (struct UgTrayIcon* trayicon)
 	gtk_menu_shell_append ((GtkMenuShell*)menu, menu_item);
 	trayicon->menu.create_download = menu_item;
 
-	// New Download from Clipboard
-	menu_item = gtk_image_menu_item_new_with_mnemonic (_("New _from Clipboard..."));
+	// New Clipboard batch
+	menu_item = gtk_image_menu_item_new_with_mnemonic (_("New Clipboard _batch..."));
 	image = gtk_image_new_from_stock (GTK_STOCK_PASTE, GTK_ICON_SIZE_MENU);
 	gtk_image_menu_item_set_image ((GtkImageMenuItem*)menu_item, image);
 	gtk_menu_shell_append ((GtkMenuShell*)menu, menu_item);
@@ -164,24 +165,25 @@ static void ug_trayicon_init (struct UgTrayIcon* trayicon)
 	                         "hicolor", "16x16", "apps",
 	                         "uget-icon.png", NULL);
 	if (g_file_test (file_name, G_FILE_TEST_IS_REGULAR))
-		icon_name = UG_APP_GTK_ICON_NAME;
+		icon_name = UG_APP_GTK_TRAY_ICON_NAME;
 	else
 		icon_name = GTK_STOCK_GO_DOWN;
 	g_free (file_name);
 #ifdef HAVE_APP_INDICATOR
 	trayicon->indicator = app_indicator_new ("uget-gtk", icon_name,
 			APP_INDICATOR_CATEGORY_APPLICATION_STATUS);
-	if (trayicon->indicator == NULL)
-		return;
-	app_indicator_set_menu (trayicon->indicator, GTK_MENU (trayicon->menu.self));
-//	app_indicator_set_attention_icon_full (trayicon->indicator,
-//			UG_APP_GTK_ICON_ACTIVE_NAME, NULL);
-	app_indicator_set_attention_icon (trayicon->indicator,
-			UG_APP_GTK_ICON_ACTIVE_NAME);
-#else
+	if (trayicon->indicator) {
+		app_indicator_set_menu (trayicon->indicator, GTK_MENU (trayicon->menu.self));
+//		app_indicator_set_attention_icon_full (trayicon->indicator,
+//				UG_APP_GTK_TRAY_ICON_ACTIVE_NAME, NULL);
+		app_indicator_set_attention_icon (trayicon->indicator,
+				UG_APP_GTK_TRAY_ICON_ACTIVE_NAME);
+		app_indicator_set_status (trayicon->indicator,
+				APP_INDICATOR_STATUS_PASSIVE);
+	}
+#endif	// HAVE_APP_INDICATOR
 	trayicon->self = gtk_status_icon_new_from_icon_name (icon_name);
 	gtk_status_icon_set_visible (trayicon->self, FALSE);
-#endif
 	ug_trayicon_set_info (trayicon, 0, 0, 0);
 }
 
@@ -197,22 +199,22 @@ static void ug_window_init  (struct UgWindow* window, UgAppGtk* app)
 	gtk_window_set_title (window->self, UG_APP_GTK_NAME);
 	gtk_window_set_default_size (window->self, 620, 400);
 	gtk_window_add_accel_group (window->self, app->accel_group);
-	gtk_window_set_default_icon_name (UG_APP_GTK_ICON_NAME);
+	gtk_window_set_default_icon_name (UG_APP_GTK_APP_ICON_NAME);
 
 	// top container for Main Window
-	vbox = (GtkBox*) gtk_vbox_new (FALSE, 0);
+	vbox = (GtkBox*) gtk_box_new (GTK_ORIENTATION_VERTICAL, 0);
 	gtk_container_add (GTK_CONTAINER (window->self), GTK_WIDGET (vbox));
 	gtk_box_pack_start (vbox, app->menubar.self, FALSE, FALSE, 0);
 	// right side vbox
-	rbox = (GtkBox*) gtk_vbox_new (FALSE, 0);
+	rbox = (GtkBox*) gtk_box_new (GTK_ORIENTATION_VERTICAL, 0);
 	gtk_box_pack_start (rbox, app->toolbar.self, FALSE, FALSE, 0);
 	// hpaned
-	window->hpaned = (GtkPaned*) gtk_hpaned_new ();
+	window->hpaned = (GtkPaned*) gtk_paned_new (GTK_ORIENTATION_HORIZONTAL);
 	gtk_box_pack_start (vbox, GTK_WIDGET (window->hpaned), TRUE, TRUE, 0);
 	gtk_paned_pack1 (window->hpaned, app->cwidget.self, FALSE, TRUE);
 	gtk_paned_pack2 (window->hpaned, GTK_WIDGET (rbox), TRUE, FALSE);
 	// vpaned
-	window->vpaned = (GtkPaned*) gtk_vpaned_new ();
+	window->vpaned = (GtkPaned*) gtk_paned_new (GTK_ORIENTATION_VERTICAL);
 	gtk_box_pack_start (rbox, (GtkWidget*) window->vpaned, TRUE, TRUE, 0);
 //	gtk_paned_pack1 (window->vpaned, GTK_WIDGET (app->cwidget.primary->all.self), TRUE, TRUE);
 	gtk_paned_pack2 (window->vpaned, GTK_WIDGET (app->summary.self), FALSE, TRUE);
@@ -232,10 +234,6 @@ static void ug_statusbar_init (struct UgStatusbar* sbar)
 	sbar->self = (GtkStatusbar*) gtk_statusbar_new ();
 	hbox = GTK_BOX (sbar->self);
 
-#if GTK_MAJOR_VERSION < 3
-	gtk_statusbar_set_has_resize_grip (sbar->self, FALSE);
-#endif
-
 	// upload speed label
 	widget = gtk_label_new ("");
 	sbar->up_speed = (GtkLabel*) widget;
@@ -245,7 +243,7 @@ static void ug_statusbar_init (struct UgStatusbar* sbar)
 	gtk_misc_set_alignment (GTK_MISC (widget), 0, 0.5);
 	gtk_box_pack_end (hbox, gtk_label_new ("U:"), FALSE, TRUE, 2);
 
-	gtk_box_pack_end (hbox, gtk_vseparator_new (), FALSE, TRUE, 8);
+	gtk_box_pack_end (hbox, gtk_separator_new (GTK_ORIENTATION_VERTICAL), FALSE, TRUE, 8);
 
 	// download speed label
 	widget = gtk_label_new ("");
@@ -300,20 +298,20 @@ static void ug_toolbar_init (struct UgToolbar* ugt, GtkAccelGroup* accel_group)
 	gtk_image_menu_item_set_image ((GtkImageMenuItem*)menu_item, image);
 	gtk_menu_shell_append ((GtkMenuShell*)menu, menu_item);
 	ugt->create_category = menu_item;
-	// New Batch download
-	menu_item = gtk_image_menu_item_new_with_mnemonic (_("New _Batch download..."));
-	image = gtk_image_new_from_stock (GTK_STOCK_SORT_ASCENDING, GTK_ICON_SIZE_MENU);
-	gtk_image_menu_item_set_image ((GtkImageMenuItem*)menu_item, image);
-	gtk_menu_shell_append ((GtkMenuShell*)menu, menu_item);
-	ugt->create_batch = menu_item;
-	gtk_widget_show_all (menu);
-	// New from Clipboard
-	menu_item = gtk_image_menu_item_new_with_mnemonic (_("New _from Clipboard..."));
+	// New Clipboard batch
+	menu_item = gtk_image_menu_item_new_with_mnemonic (_("New Clipboard _batch..."));
 	image = gtk_image_new_from_stock (GTK_STOCK_PASTE, GTK_ICON_SIZE_MENU);
 	gtk_image_menu_item_set_image ((GtkImageMenuItem*)menu_item, image);
 	gtk_menu_shell_append ((GtkMenuShell*)menu, menu_item);
 	ugt->create_clipboard = menu_item;
 	gtk_widget_show_all (menu);
+	// New URL Sequence batch
+	menu_item = gtk_image_menu_item_new_with_mnemonic (_("New _URL Sequence batch..."));
+	image = gtk_image_new_from_stock (GTK_STOCK_SORT_ASCENDING, GTK_ICON_SIZE_MENU);
+	gtk_image_menu_item_set_image ((GtkImageMenuItem*)menu_item, image);
+	gtk_menu_shell_append ((GtkMenuShell*)menu, menu_item);
+	ugt->create_sequence = menu_item;
+	gtk_widget_show_all (menu);
 
 	gtk_menu_shell_append ((GtkMenuShell*)menu, gtk_separator_menu_item_new() );
 
@@ -422,21 +420,8 @@ static void ug_menubar_init (struct UgMenubar* menubar, GtkAccelGroup* accel_gro
 	gtk_image_menu_item_set_image ((GtkImageMenuItem*)menu_item, image);
 	gtk_menu_shell_append ((GtkMenuShell*)sub_menu, menu_item);
 	menubar->file.create.category = menu_item;
-	// New - Batch download
-	menu_item = gtk_image_menu_item_new_with_mnemonic (_("_Batch download..."));
-	image = gtk_image_new_from_stock (GTK_STOCK_SORT_ASCENDING, GTK_ICON_SIZE_MENU);
-	gtk_image_menu_item_set_image ((GtkImageMenuItem*)menu_item, image);
-	gtk_menu_shell_append ((GtkMenuShell*)sub_menu, menu_item);
-	menubar->file.create.batch = menu_item;
-	// New - from clipboard
-	menu_item = gtk_image_menu_item_new_with_mnemonic (_("_From Clipboard..."));
-	image = gtk_image_new_from_stock (GTK_STOCK_PASTE, GTK_ICON_SIZE_MENU);
-	gtk_image_menu_item_set_image ((GtkImageMenuItem*)menu_item, image);
-	gtk_menu_shell_append ((GtkMenuShell*)sub_menu, menu_item);
-	menubar->file.create.from_clipboard = menu_item;
-
+	// New - separator
 	gtk_menu_shell_append ((GtkMenuShell*)sub_menu, gtk_separator_menu_item_new() );
-
 	// New - Torrent
 	menu_item = gtk_image_menu_item_new_with_mnemonic (_("Torrent..."));
 	image = gtk_image_new_from_stock (GTK_STOCK_FILE, GTK_ICON_SIZE_MENU);
@@ -451,6 +436,46 @@ static void ug_menubar_init (struct UgMenubar* menubar, GtkAccelGroup* accel_gro
 	menubar->file.create.metalink = menu_item;
 	// New --- end ---
 
+	// Batch Downloads --- start ---
+	menu_item = gtk_image_menu_item_new_with_mnemonic (_("_Batch Downloads"));
+	sub_menu = gtk_menu_new ();
+	gtk_menu_set_accel_group ((GtkMenu*)sub_menu, accel_group);
+	gtk_menu_item_set_submenu ((GtkMenuItem*)menu_item, sub_menu);
+	gtk_menu_shell_append ((GtkMenuShell*)menu, menu_item);
+	// Batch downloads - Clipboard batch
+	menu_item = gtk_image_menu_item_new_with_mnemonic (_("_Clipboard batch..."));
+	image = gtk_image_new_from_stock (GTK_STOCK_PASTE, GTK_ICON_SIZE_MENU);
+	gtk_image_menu_item_set_image ((GtkImageMenuItem*)menu_item, image);
+	gtk_menu_shell_append ((GtkMenuShell*)sub_menu, menu_item);
+	menubar->file.batch.clipboard = menu_item;
+	// Batch downloads - URL Sequence batch
+	menu_item = gtk_image_menu_item_new_with_mnemonic (_("_URL Sequence batch..."));
+	image = gtk_image_new_from_stock (GTK_STOCK_SORT_ASCENDING, GTK_ICON_SIZE_MENU);
+	gtk_image_menu_item_set_image ((GtkImageMenuItem*)menu_item, image);
+	gtk_menu_shell_append ((GtkMenuShell*)sub_menu, menu_item);
+	menubar->file.batch.sequence = menu_item;
+	// Batch downloads - Text file import (.txt)
+	menu_item = gtk_image_menu_item_new_with_mnemonic (_("_Text file import (.txt)..."));
+	image = gtk_image_new_from_stock (GTK_STOCK_GO_FORWARD, GTK_ICON_SIZE_MENU);
+	gtk_image_menu_item_set_image ((GtkImageMenuItem*)menu_item, image);
+	gtk_menu_shell_append ((GtkMenuShell*)sub_menu, menu_item);
+	menubar->file.batch.text_import = menu_item;
+	// Batch downloads - HTML file import (.html)
+	menu_item = gtk_image_menu_item_new_with_mnemonic (_("_HTML file import (.html)..."));
+	image = gtk_image_new_from_stock (GTK_STOCK_CONVERT, GTK_ICON_SIZE_MENU);
+	gtk_image_menu_item_set_image ((GtkImageMenuItem*)menu_item, image);
+	gtk_menu_shell_append ((GtkMenuShell*)sub_menu, menu_item);
+	menubar->file.batch.html_import = menu_item;
+	// Batch downloads - separator
+	gtk_menu_shell_append ((GtkMenuShell*)sub_menu, gtk_separator_menu_item_new() );
+	// Batch downloads - Export to Text file (.txt)
+	menu_item = gtk_image_menu_item_new_with_mnemonic (_("_Export to Text file (.txt)..."));
+	image = gtk_image_new_from_stock (GTK_STOCK_GO_BACK, GTK_ICON_SIZE_MENU);
+	gtk_image_menu_item_set_image ((GtkImageMenuItem*)menu_item, image);
+	gtk_menu_shell_append ((GtkMenuShell*)sub_menu, menu_item);
+	menubar->file.batch.text_export = menu_item;
+	// Batch downloads --- end ---
+
 	gtk_menu_shell_append ((GtkMenuShell*)menu, gtk_separator_menu_item_new() );
 
 	// Save
@@ -460,26 +485,6 @@ static void ug_menubar_init (struct UgMenubar* menubar, GtkAccelGroup* accel_gro
 
 	gtk_menu_shell_append ((GtkMenuShell*)menu, gtk_separator_menu_item_new() );
 
-	menu_item = gtk_image_menu_item_new_with_mnemonic (_("Import _HTML file..."));
-	image = gtk_image_new_from_stock (GTK_STOCK_CONVERT, GTK_ICON_SIZE_MENU);
-	gtk_image_menu_item_set_image ((GtkImageMenuItem*)menu_item, image);
-	gtk_menu_shell_append ((GtkMenuShell*)menu, menu_item);
-	menubar->file.import_html = menu_item;
-
-	menu_item = gtk_image_menu_item_new_with_mnemonic (_("_Import text file..."));
-	image = gtk_image_new_from_stock (GTK_STOCK_GO_FORWARD, GTK_ICON_SIZE_MENU);
-	gtk_image_menu_item_set_image ((GtkImageMenuItem*)menu_item, image);
-	gtk_menu_shell_append ((GtkMenuShell*)menu, menu_item);
-	menubar->file.import_text = menu_item;
-
-	menu_item = gtk_image_menu_item_new_with_mnemonic (_("_Export text file..."));
-	image = gtk_image_new_from_stock (GTK_STOCK_GO_BACK, GTK_ICON_SIZE_MENU);
-	gtk_image_menu_item_set_image ((GtkImageMenuItem*)menu_item, image);
-	gtk_menu_shell_append ((GtkMenuShell*)menu, menu_item);
-	menubar->file.export_text = menu_item;
-
-	gtk_menu_shell_append ((GtkMenuShell*)menu, gtk_separator_menu_item_new() );
-
 	// Offline mode
 	menu_item = gtk_check_menu_item_new_with_mnemonic (_("_Offline Mode"));
 	gtk_menu_shell_append ((GtkMenuShell*)menu, menu_item);
@@ -696,6 +701,7 @@ static void ug_menubar_init (struct UgMenubar* menubar, GtkAccelGroup* accel_gro
 	// ----------------------------------------------------
 	// UgDownloadMenu
 	menu = gtk_menu_new ();
+	gtk_menu_set_accel_group ((GtkMenu*)menu, accel_group);
 	menu_item = gtk_menu_item_new_with_mnemonic (_("_Download"));
 	gtk_menu_item_set_submenu ((GtkMenuItem*)menu_item, menu);
 	gtk_menu_shell_append ((GtkMenuShell*)menubar->self, menu_item);
@@ -707,7 +713,8 @@ static void ug_menubar_init (struct UgMenubar* menubar, GtkAccelGroup* accel_gro
 	gtk_menu_shell_append ((GtkMenuShell*)menu, menu_item);
 	menubar->download.create = menu_item;
 
-	menu_item = gtk_image_menu_item_new_from_stock (GTK_STOCK_DELETE, NULL);
+	menu_item = gtk_image_menu_item_new_from_stock (GTK_STOCK_DELETE, accel_group);
+	gtk_menu_item_set_accel_path ((GtkMenuItem*) menu_item, UG_APP_GTK_ACCEL_PATH_DELETE);
 //	menu_item = gtk_image_menu_item_new_with_mnemonic (_("_Delete"));
 //	image = gtk_image_new_from_stock (GTK_STOCK_DELETE, GTK_ICON_SIZE_MENU);
 //	gtk_image_menu_item_set_image ((GtkImageMenuItem*)menu_item, image);
@@ -715,6 +722,7 @@ static void ug_menubar_init (struct UgMenubar* menubar, GtkAccelGroup* accel_gro
 	menubar->download.delete = menu_item;
 
 	menu_item = gtk_image_menu_item_new_with_mnemonic (_("Delete _File and Data"));
+	gtk_menu_item_set_accel_path ((GtkMenuItem*) menu_item, UG_APP_GTK_ACCEL_PATH_DELETE_F);
 //	image = gtk_image_new_from_stock (GTK_STOCK_DELETE, GTK_ICON_SIZE_MENU);
 //	gtk_image_menu_item_set_image ((GtkImageMenuItem*)menu_item, image);
 	gtk_menu_shell_append ((GtkMenuShell*)menu, menu_item);
@@ -723,11 +731,13 @@ static void ug_menubar_init (struct UgMenubar* menubar, GtkAccelGroup* accel_gro
 	gtk_menu_shell_append ((GtkMenuShell*)menu, gtk_separator_menu_item_new() );
 
 	menu_item = gtk_image_menu_item_new_from_stock (GTK_STOCK_OPEN, NULL);
+	gtk_menu_item_set_accel_path ((GtkMenuItem*) menu_item, UG_APP_GTK_ACCEL_PATH_OPEN);
 	gtk_menu_shell_append ((GtkMenuShell*)menu, menu_item);
 	menubar->download.open = menu_item;
 	gtk_widget_hide (menu_item);
 
 	menu_item = gtk_image_menu_item_new_with_mnemonic(_("Open _containing folder"));
+	gtk_menu_item_set_accel_path ((GtkMenuItem*) menu_item, UG_APP_GTK_ACCEL_PATH_OPEN_F);
 	image = gtk_image_new_from_stock (GTK_STOCK_DIRECTORY, GTK_ICON_SIZE_MENU);
 	gtk_image_menu_item_set_image ((GtkImageMenuItem*)menu_item, image);
 	gtk_menu_shell_append ((GtkMenuShell*)menu, menu_item);
