@@ -1,6 +1,6 @@
 /*
  *
- *   Copyright (C) 2005-2012 by C.H. Huang
+ *   Copyright (C) 2005-2013 by C.H. Huang
  *   plushuang.tw@gmail.com
  *
  *  This library is free software; you can redistribute it and/or
@@ -38,7 +38,7 @@
 #ifdef _WIN32
 // This is for ATTACH_PARENT_PROCESS
 #ifndef _WIN32_WINNT
-#define _WIN32_WINNT 0x501
+#define _WIN32_WINNT 0x501	// WinXP
 #endif
 #define _CRT_SECURE_NO_WARNINGS		1
 #include <windows.h>
@@ -134,7 +134,7 @@ gboolean	gst_inited	= FALSE;
 // ug_get_data_dir() defined in UgApp-gtk.h
 const gchar*	ug_get_data_dir (void)
 {
-#ifdef _WIN32
+#ifdef DATADIR_WIN_PROG		// change DATADIR to main program installed path.
 	static	gchar*	data_dir = NULL;
 
 	if (data_dir == NULL) {
@@ -156,7 +156,7 @@ const gchar*	ug_get_data_dir (void)
 	return DATADIR;
 #else
 	return "/usr/share";
-#endif  // _WIN32
+#endif  // DATADIR_WIN_PROG
 }
 
 // ----------------------------------------------------------------------------
@@ -213,16 +213,11 @@ int main (int argc, char** argv)
 		ug_option_help (&app->option, argv[0], string);
 	}
 
-	// GLib: GThread
-	if (g_thread_supported () == FALSE)
-		g_thread_init (NULL);
 	// GTK+
 	gtk_init (&argc, &argv);
 
 	// GStreamer
 #ifdef HAVE_GSTREAMER
-	// initialize the GLib thread system using g_thread_init()
-	// before any other GLib functions are called.
 	gst_inited = gst_init_check (&argc, &argv, NULL);
 #endif
 
@@ -247,9 +242,7 @@ int main (int argc, char** argv)
 	// main program
 	ug_app_init (app);
 	signal (SIGTERM, term_signal_handler);
-	gdk_threads_enter ();
 	gtk_main ();
-	gdk_threads_leave ();
 
 	// libnotify
 #ifdef HAVE_LIBNOTIFY
