@@ -1,6 +1,6 @@
 /*
  *
- *   Copyright (C) 2005-2012 by C.H. Huang
+ *   Copyright (C) 2005-2013 by C.H. Huang
  *   plushuang.tw@gmail.com
  *
  *  This library is free software; you can redistribute it and/or
@@ -356,7 +356,9 @@ static gboolean	ug_app_timeout_ipc (UgAppGtk* app)
 		g_list_foreach (list->next, (GFunc) ug_download_assign_attachment, list->data);
 	// add downloads
 	category_index = app->option.data->category_index;
-	if (app->option.data->quiet)
+	if (category_index == -1)
+		category_index = app->setting.commandline.category_index;
+	if (app->option.data->quiet || app->setting.commandline.quiet)
 		uget_add_download_quietly (app, list, category_index);
 	else
 		uget_add_download_selected (app, list, category_index);
@@ -426,8 +428,8 @@ static gboolean	ug_app_decide_schedule_state (UgAppGtk* app)
 			break;
 
 		case UG_SCHEDULE_LIMITED_SPEED:
-			ug_running_set_speed (&app->running,
-					app->setting.scheduler.speed_limit, 0);
+//			ug_running_set_speed (&app->running,
+//					app->setting.speed_limit.scheduler.download, 0);
 			break;
 
 		default:
@@ -456,7 +458,12 @@ static gboolean	ug_app_timeout_queuing (UgAppGtk* app)
 		gchar*		string;
 	} temp;
 
-
+	// check aria2.failed_count (one shot)
+	if (app->aria2.failed_count == 5) {
+		ug_app_show_message (app, GTK_MESSAGE_ERROR,
+				_("Failed to connect to aria2.\n"
+				  "Is aria2 shutdown? or aria2 launch failed."));
+	}
 	// If changed is TRUE, it will refresh all category-related data.
 	changed = ug_app_decide_schedule_state (app);
 	// do something for inactive tasks
@@ -504,7 +511,7 @@ static gboolean	ug_app_timeout_queuing (UgAppGtk* app)
 		}
 		// downloading completed
 		else if (n_before > 0  &&  n_after == 0) {
-			if (app->user_action == FALSE) {
+			if (app->action.stop == FALSE) {
 				// completed notification
 				ug_app_notify_completed (app);
 				// shutdown
@@ -530,7 +537,14 @@ static gboolean	ug_app_timeout_queuing (UgAppGtk* app)
 
 	// category or download status changed
 	if (changed || n_after) {
-		ug_running_get_speed (&app->running, &down_speed, &up_speed);
+		// get overall speed
+//		if (app->setting.plugin.aria2.enable == FALSE)
+			ug_running_get_speed (&app->running, &down_speed, &up_speed);
+//		else {
+//			down_speed = app->aria2.download_speed;
+//			up_speed   = app->aria2.upload_speed;
+//		}
+		// refresh download info
 		gtk_widget_queue_draw (app->cwidget.current.widget->self);
 		// summary
 		ug_summary_show (&app->summary,
@@ -544,6 +558,8 @@ static gboolean	ug_app_timeout_queuing (UgAppGtk* app)
 	if (changed)
 		gtk_widget_queue_draw (app->cwidget.self);
 
+	// reset action status "stop by user"
+	app->action.stop = FALSE;
 	// return FALSE if the source should be removed.
 	return TRUE;
 }
@@ -656,16 +672,16 @@ static void ug_app_notify (UgAppGtk* app, const gchar* title, const gchar* body)
 
 #if defined (NOTIFY_VERSION_MINOR) && NOTIFY_VERSION_MAJOR >= 0 && NOTIFY_VERSION_MINOR >= 7
 		notification = notify_notification_new (string,
-				body, UG_APP_GTK_ICON_NAME);
+				body, UG_APP_GTK_APP_ICON_NAME);
 #else
 		notification = notify_notification_new (string,
-				body, UG_APP_GTK_ICON_NAME, NULL);
+				body, UG_APP_GTK_APP_ICON_NAME, NULL);
 #endif
 		notify_notification_set_timeout (notification, 7000);	// milliseconds
 	}
 	else {
 		notify_notification_update (notification, string,
-				body, UG_APP_GTK_ICON_NAME);
+				body, UG_APP_GTK_TRAY_ICON_NAME);
 	}
 	g_free (string);
 
