@@ -1,6 +1,6 @@
 /*
  *
- *   Copyright (C) 2005-2012 by C.H. Huang
+ *   Copyright (C) 2005-2013 by C.H. Huang
  *   plushuang.tw@gmail.com
  *
  *  This library is free software; you can redistribute it and/or
@@ -106,7 +106,7 @@ void	ug_app_init (UgAppGtk* app)
 	ug_app_init_callback (app);
 	// initialize timeout in UgApp-gtk-timeout.c
 	ug_app_init_timeout (app);
-	// aria2
+	// initialize aria2
 	ug_app_aria2_init (app);
 	ug_app_aria2_setup (app);
 
@@ -131,13 +131,15 @@ void	ug_app_quit (UgAppGtk* app)
 	ug_app_get_setting (app, &app->setting);
 	// save data
 	ug_app_save (app);
+	// aria2
+	ug_app_aria2_finalize (app);
 	// hide icon in system tray before quit
 	ug_trayicon_set_visible (&app->trayicon, FALSE);
 	// hide window
 	gtk_widget_hide (GTK_WIDGET (app->window.self));
 	// aria2
 	ug_app_aria2_shutdown (app);
-	// This will quit  gtk_main()  to  main()  in  main-gtk.c
+	// This will quit gtk_main() in main()
 	gtk_main_quit ();
 }
 
@@ -230,6 +232,12 @@ void	ug_app_set_setting (UgAppGtk* app, UgSetting* setting)
 			setting->window.category);
 	gtk_widget_set_visible (app->summary.self,
 			setting->window.summary);
+	// Summary
+	app->summary.visible.name     = setting->summary.name;
+	app->summary.visible.folder   = setting->summary.folder;
+	app->summary.visible.category = setting->summary.category;
+	app->summary.visible.url      = setting->summary.url;
+	app->summary.visible.message  = setting->summary.message;
 	// ----------------------------------------------------
 	// UgEditMenu
 	gtk_check_menu_item_set_active (
@@ -309,6 +317,12 @@ void	ug_app_set_setting (UgAppGtk* app, UgSetting* setting)
 			(GtkCheckMenuItem*) app->menubar.view.columns.completed_on,
 			setting->download_column.completed_on);
 
+#ifdef HAVE_APP_INDICATOR
+	// AppIndicator
+	if (setting->ui.app_indicator == FALSE)
+		app->trayicon.indicator = NULL;
+#endif	// HAVE_APP_INDICATOR
+
 	// get setting of download column
 	app->cwidget.sort.nth   = app->setting.download_column.sort.nth;
 	app->cwidget.sort.order = app->setting.download_column.sort.order;
@@ -445,44 +459,42 @@ void	ug_trayicon_set_info (struct UgTrayIcon* trayicon, guint n_active, gint64 d
 	gchar*	string_up_speed;
 	guint	current_status;
 
-#ifdef HAVE_APP_INDICATOR
-	if (trayicon->indicator == NULL)
-		return;
-	trayicon->error_occurred = FALSE;
-#endif
 	// change tray icon
 	if (trayicon->error_occurred) {
-		string = UG_APP_GTK_ICON_ERROR_NAME;
+		string = UG_APP_GTK_TRAY_ICON_ERROR_NAME;
 		current_status = 2;
 	}
 	else if (n_active > 0) {
-		string = UG_APP_GTK_ICON_ACTIVE_NAME;
+		string = UG_APP_GTK_TRAY_ICON_ACTIVE_NAME;
 		current_status = 1;
 	}
 	else {
-		string = UG_APP_GTK_ICON_NAME;
+		string = UG_APP_GTK_TRAY_ICON_NAME;
 		current_status = 0;
 	}
 
 	if (trayicon->last_status != current_status) {
 		trayicon->last_status  = current_status;
 #ifdef HAVE_APP_INDICATOR
-		if (app_indicator_get_status (trayicon->indicator) != APP_INDICATOR_STATUS_PASSIVE) {
-			if (current_status == 0) {
-				app_indicator_set_status (trayicon->indicator,
-						APP_INDICATOR_STATUS_ACTIVE);
-			}
-			else {
-				app_indicator_set_attention_icon (trayicon->indicator, string);
-//				app_indicator_set_attention_icon_full (trayicon->indicator,
-//						string, "attention");
-				app_indicator_set_status (trayicon->indicator,
-						APP_INDICATOR_STATUS_ATTENTION);
+		if (trayicon->indicator) {
+			trayicon->error_occurred = FALSE;
+			if (app_indicator_get_status (trayicon->indicator) != APP_INDICATOR_STATUS_PASSIVE) {
+				if (current_status == 0) {
+					app_indicator_set_status (trayicon->indicator,
+							APP_INDICATOR_STATUS_ACTIVE);
+				}
+				else {
+					app_indicator_set_attention_icon (trayicon->indicator, string);
+	//				app_indicator_set_attention_icon_full (trayicon->indicator,
+	//						string, "attention");
+					app_indicator_set_status (trayicon->indicator,
+							APP_INDICATOR_STATUS_ATTENTION);
+				}
 			}
 		}
-#else
+		else
+#endif	// HAVE_APP_INDICATOR
 		gtk_status_icon_set_from_icon_name (trayicon->self, string);
-#endif
 	}
 
 	// change tooltip
@@ -497,11 +509,14 @@ void	ug_trayicon_set_info (struct UgTrayIcon* trayicon, guint n_active, gint64 d
 			string_down_speed,
 			string_up_speed);
 #ifdef HAVE_APP_INDICATOR
-//	g_object_set (trayicon->indicator, "icon-desc", string, NULL);
-//	g_object_set (trayicon->indicator, "attention-icon-desc", string, NULL);
-#else
+	if (trayicon->indicator) {
+//		g_object_set (trayicon->indicator, "icon-desc", string, NULL);
+//		g_object_set (trayicon->indicator, "attention-icon-desc", string, NULL);
+	}
+	else
+#endif	// HAVE_APP_INDICATOR
 	gtk_status_icon_set_tooltip_text (trayicon->self, string);
-#endif
+
 	g_free (string_down_speed);
 	g_free (string_up_speed);
 	g_free (string);
@@ -510,16 +525,17 @@ void	ug_trayicon_set_info (struct UgTrayIcon* trayicon, guint n_active, gint64 d
 void	ug_trayicon_set_visible (struct UgTrayIcon* trayicon, gboolean visible)
 {
 #ifdef HAVE_APP_INDICATOR
-	if (trayicon->indicator == NULL)
-		return;
-
-	if (visible)
-		app_indicator_set_status (trayicon->indicator, APP_INDICATOR_STATUS_ACTIVE);
+	if (trayicon->indicator) {
+		if (visible)
+			app_indicator_set_status (trayicon->indicator,
+					APP_INDICATOR_STATUS_ACTIVE);
+		else
+			app_indicator_set_status (trayicon->indicator,
+					APP_INDICATOR_STATUS_PASSIVE);
+	}
 	else
-		app_indicator_set_status (trayicon->indicator, APP_INDICATOR_STATUS_PASSIVE);
-#else
+#endif	// HAVE_APP_INDICATOR
 	gtk_status_icon_set_visible (trayicon->self, visible);
-#endif
 }
 
 void	ug_statusbar_set_info (struct UgStatusbar* statusbar, UgDownloadWidget* dwidget)
@@ -618,14 +634,14 @@ void	ug_app_confirm_to_quit (UgAppGtk* app)
 	gtk_container_set_border_width (GTK_CONTAINER (dialog), 4);
 	vbox = (GtkBox*) gtk_dialog_get_content_area (GTK_DIALOG (dialog));
 	// image and label
-	hbox = (GtkBox*) gtk_hbox_new (FALSE, 2);
+	hbox = (GtkBox*) gtk_box_new (GTK_ORIENTATION_HORIZONTAL, 2);
 	gtk_box_pack_start (hbox, gtk_image_new_from_stock (GTK_STOCK_DIALOG_QUESTION, GTK_ICON_SIZE_DIALOG),
 	                    FALSE, FALSE, 8);
 	gtk_box_pack_start (hbox, gtk_label_new (_("Are you sure you want to quit?")),
 	                    FALSE, FALSE, 4);
 	gtk_box_pack_start (vbox, (GtkWidget*) hbox, FALSE, FALSE, 6);
 	// check button
-	hbox = (GtkBox*) gtk_hbox_new (FALSE, 2);
+	hbox = (GtkBox*) gtk_box_new (GTK_ORIENTATION_HORIZONTAL, 2);
 	button = gtk_check_button_new_with_label (_("Remember this action"));
 	gtk_box_pack_end (hbox, button, TRUE, TRUE, 20);
 	gtk_box_pack_end (vbox, (GtkWidget*) hbox, FALSE, FALSE, 10);
@@ -703,14 +719,14 @@ void	ug_app_confirm_to_delete (UgAppGtk* app, GCallback response, gpointer respo
 	gtk_container_set_border_width (GTK_CONTAINER (dialog), 4);
 	vbox = (GtkBox*) gtk_dialog_get_content_area (GTK_DIALOG (dialog));
 	// image and label
-	hbox = (GtkBox*) gtk_hbox_new (FALSE, 2);
+	hbox = (GtkBox*) gtk_box_new (GTK_ORIENTATION_HORIZONTAL, 2);
 	gtk_box_pack_start (hbox, gtk_image_new_from_stock (GTK_STOCK_DIALOG_QUESTION, GTK_ICON_SIZE_DIALOG),
 	                    FALSE, FALSE, 8);
 	gtk_box_pack_start (hbox, gtk_label_new (_("Are you sure you want to delete files?")),
 	                    FALSE, FALSE, 4);
 	gtk_box_pack_start (vbox, (GtkWidget*) hbox, FALSE, FALSE, 6);
 	// check button
-	hbox = (GtkBox*) gtk_hbox_new (FALSE, 2);
+	hbox = (GtkBox*) gtk_box_new (GTK_ORIENTATION_HORIZONTAL, 2);
 	button = gtk_check_button_new_with_label (_("Do not ask me again"));
 	gtk_box_pack_end (hbox, button, TRUE, TRUE, 20);
 	gtk_box_pack_end (vbox, (GtkWidget*) hbox, FALSE, FALSE, 10);
@@ -755,12 +771,14 @@ void	ug_app_show_message (UgAppGtk* app, GtkMessageType type, const gchar* messa
 		value = &app->dialogs.error;
 		title = g_strconcat (UG_APP_GTK_NAME " - ", _("Error"), NULL);
 		break;
+
 	default:
 		if (app->dialogs.message)
 			gtk_widget_destroy (app->dialogs.message);
 		app->dialogs.message = dialog;
 		value = &app->dialogs.message;
 		title = g_strconcat (UG_APP_GTK_NAME " - ", _("Message"), NULL);
+		break;
 	}
 	gtk_window_set_title ((GtkWindow*) dialog, title);
 	g_free (title);
