@@ -1,6 +1,6 @@
 /*
  *
- *   Copyright (C) 2005-2012 by C.H. Huang
+ *   Copyright (C) 2005-2013 by C.H. Huang
  *   plushuang.tw@gmail.com
  *
  *  This library is free software; you can redistribute it and/or
@@ -61,10 +61,15 @@ extern "C" {
 #define UG_APP_GTK_SETTING_FILE			"Setting.xml"
 #define UG_APP_GTK_CATEGORY_FILE		"CategoryList.xml"
 #define UG_APP_GTK_DOWNLOAD_FILE		"DownloadList.xml"
-#define UG_APP_GTK_ICON_NAME			"uget-icon"
-#define UG_APP_GTK_ICON_ERROR_NAME		"uget-error"
-#define UG_APP_GTK_ICON_ACTIVE_NAME		"uget-downloading"
-#define	UG_APP_GTK_ACCEL_PATH_CTRL_N	"<uGet>/New/Download"
+#define UG_APP_GTK_APP_ICON_NAME		"uget-icon"
+#define UG_APP_GTK_TRAY_ICON_NAME			"uget-tray-default"
+#define UG_APP_GTK_TRAY_ICON_ERROR_NAME		"uget-tray-error"
+#define UG_APP_GTK_TRAY_ICON_ACTIVE_NAME	"uget-tray-downloading"
+#define	UG_APP_GTK_ACCEL_PATH_CTRL_N	"<uGet>/Download/New"
+#define	UG_APP_GTK_ACCEL_PATH_DELETE	"<uGet>/Download/Delete"
+#define	UG_APP_GTK_ACCEL_PATH_DELETE_F	"<uGet>/Download/DeleteFile"
+#define	UG_APP_GTK_ACCEL_PATH_OPEN		"<uGet>/Download/Open"
+#define	UG_APP_GTK_ACCEL_PATH_OPEN_F	"<uGet>/Download/OpenFolder"
 #define	UG_APP_GTK_CATEGORY_STOCK		GTK_STOCK_DND_MULTIPLE
 
 typedef struct	UgAppGtk				UgAppGtk;
@@ -86,18 +91,38 @@ struct UgAppGtk
 	UgRunning		running;
 
 	UgSetting		setting;		// UgSetting.h
-	gboolean		user_action;	// some task stop by user
 	UgScheduleState	schedule_state;
 
-	// Launch application
-	GRegex*			launch_regex;
+	GRegex*			launch_regex;	// for Launch application
+
+	// last download settings
+	struct LastDownloadSetting
+	{
+		UgDataset*		download;
+		gint			category_index;
+	} last;
+
 	// aria2 plug-in
 	struct
 	{
-		UgXmlrpc	xmlrpc;
+		GThread*	thread;
+		GMutex		mutex;
+		gchar*		uri;
+		// status
 		gboolean	launched;
+		gboolean	remote_updated;
+		guint		failed_count;
+		// overall speed
+		gint64		download_speed;
+		gint64		upload_speed;
 	} aria2;
 
+	struct UgUserAction
+	{
+		gboolean	deleted;		// some task deleted by user
+		gboolean	stop;			// some task stop by user
+	} action;
+
 	// Clipboard
 	struct UgClipboard
 	{
@@ -128,9 +153,8 @@ struct UgAppGtk
 	{
 #ifdef HAVE_APP_INDICATOR
 		AppIndicator*	indicator;
-#else
-		GtkStatusIcon*	self;
 #endif
+		GtkStatusIcon*	self;
 		gboolean		error_occurred;
 		guint			last_status;
 
@@ -182,8 +206,8 @@ struct UgAppGtk
 		// menu for tool button
 		GtkWidget*		create_download;
 		GtkWidget*		create_category;
-		GtkWidget*		create_batch;
 		GtkWidget*		create_clipboard;
+		GtkWidget*		create_sequence;
 		GtkWidget*		create_torrent;
 		GtkWidget*		create_metalink;
 
@@ -214,18 +238,21 @@ struct UgAppGtk
 			{
 				GtkWidget*	download;
 				GtkWidget*	category;
-				GtkWidget*	batch;
-				GtkWidget*	from_clipboard;
 				GtkWidget*	torrent;
 				GtkWidget*	metalink;
 			} create;
-			// file.save
-			GtkWidget*	save;
-			// file.import & export
-			GtkWidget*	import_html;
-			GtkWidget*	import_text;
-			GtkWidget*	export_text;
 
+			// file.batch
+			struct UgFileBatchMenu
+			{
+				GtkWidget*	clipboard;		// Clipboard batch
+				GtkWidget*	sequence;		// URL Sequence batch
+				GtkWidget*	text_import;	// Text file import (.txt)
+				GtkWidget*	html_import;	// HTML file import (.html)
+				GtkWidget*	text_export;	// Export to Text file (.txt)
+			} batch;
+
+			GtkWidget*	save;
 			GtkWidget*	offline_mode;
 			GtkWidget*	quit;
 		} file;
@@ -373,11 +400,13 @@ void	ug_app_decide_bt_meta_sensitive (UgAppGtk* app);
 // aria2
 #ifdef HAVE_PLUGIN_ARIA2
 void		ug_app_aria2_init (UgAppGtk* app);
+void		ug_app_aria2_finalize (UgAppGtk* app);
 gboolean	ug_app_aria2_setup (UgAppGtk* app);
 gboolean	ug_app_aria2_launch (UgAppGtk* app);
 void		ug_app_aria2_shutdown (UgAppGtk* app);
 #else
 #define		ug_app_aria2_init(app)
+#define		ug_app_aria2_finalize(app)
 #define		ug_app_aria2_setup(app)
 #define		ug_app_aria2_launch(app)
 #define		ug_app_aria2_shutdown(app)
