@@ -1,6 +1,6 @@
 /*
  *
- *   Copyright (C) 2005-2012 by C.H. Huang
+ *   Copyright (C) 2005-2013 by C.H. Huang
  *   plushuang.tw@gmail.com
  *
  *  This library is free software; you can redistribute it and/or
@@ -48,7 +48,7 @@ typedef	void	(*UgBatchFormNotify) (gpointer user_data, gboolean completed);
 
 struct UgBatchForm
 {
-	GtkWidget*	self;		// GtkTable
+	GtkWidget*	self;		// GtkGrid
 
 	GtkEntry*	entry;		// URL, wildcard
 	GtkWidget*	radio;		// GtkRadioButton
