@@ -1,6 +1,6 @@
 /*
  *
- *   Copyright (C) 2005-2012 by C.H. Huang
+ *   Copyright (C) 2005-2013 by C.H. Huang
  *   plushuang.tw@gmail.com
  *
  *  This library is free software; you can redistribute it and/or
@@ -45,15 +45,6 @@
 
 #include <glib/gi18n.h>
 
-// for GTK+ 2.18
-#ifndef GTK_COMBO_BOX_TEXT
-#define	GTK_COMBO_BOX_TEXT					GTK_COMBO_BOX
-#define	GtkComboBoxText						GtkComboBox
-#define	gtk_combo_box_text_new				gtk_combo_box_new_text
-#define	gtk_combo_box_text_new_with_entry	gtk_combo_box_entry_new_text
-#define	gtk_combo_box_text_append_text		gtk_combo_box_append_text
-#endif	// GTK_COMBO_BOX_TEXT
-
 
 static void	ug_download_form_init_page1 (UgDownloadForm* dform, UgProxyForm* proxy);
 static void	ug_download_form_init_page2 (UgDownloadForm* dform);
@@ -79,6 +70,8 @@ void	ug_download_form_init (UgDownloadForm* dform, UgProxyForm* proxy, GtkWindow
 	dform->changed.referrer = FALSE;
 	dform->changed.cookie   = FALSE;
 	dform->changed.post     = FALSE;
+	dform->changed.agent    = FALSE;
+	dform->changed.timestamp= FALSE;
 	dform->changed.retry    = FALSE;
 	dform->changed.delay    = FALSE;
 	dform->parent = parent;
@@ -113,45 +106,68 @@ GtkWidget*	ug_download_from_use_notebook (UgDownloadForm* dform, const gchar* la
 static void	ug_download_form_init_page1 (UgDownloadForm* dform, UgProxyForm* proxy)
 {
 	GtkWidget*	widget;
-	GtkTable*	top_table;
-	GtkTable*	table;
+	GtkGrid*	top_grid;
+	GtkGrid*	grid;
 	GtkWidget*	frame;
 	GtkWidget*	vbox;
 	GtkWidget*	hbox;
 
-	dform->page1 = gtk_table_new (7, 3, FALSE);
-	top_table = (GtkTable*) dform->page1;
-	gtk_container_set_border_width (GTK_CONTAINER (top_table), 2);
+	dform->page1 = gtk_grid_new ();
+	top_grid = (GtkGrid*) dform->page1;
+	gtk_container_set_border_width (GTK_CONTAINER (top_grid), 2);
 
 	// URL - entry
 	widget = gtk_entry_new ();
 	gtk_entry_set_width_chars (GTK_ENTRY (widget), 20);
 	gtk_entry_set_activates_default (GTK_ENTRY (widget), TRUE);
-	gtk_table_attach (top_table, widget,  1, 3, 0, 1,
-			GTK_FILL | GTK_EXPAND, GTK_SHRINK, 1, 2);
+	g_object_set (widget, "margin-left", 1, "margin-right", 1, NULL);
+	g_object_set (widget, "margin-top", 2, "margin-bottom", 2, NULL);
+	g_object_set (widget, "hexpand", TRUE, NULL);
+	gtk_grid_attach (top_grid, widget, 1, 0, 2, 1);
 	g_signal_connect (GTK_EDITABLE (widget), "changed",
 			G_CALLBACK (on_url_entry_changed), dform);
 	dform->url_entry = widget;
 	// URL - label
 	widget = gtk_label_new_with_mnemonic (_("_URL:"));
 	gtk_label_set_mnemonic_widget (GTK_LABEL(widget), dform->url_entry);
-	gtk_table_attach (top_table, widget,  0, 1, 0, 1,
-			GTK_SHRINK, GTK_SHRINK, 3, 2);
+	g_object_set (widget, "margin-left", 3, "margin-right", 3, NULL);
+	g_object_set (widget, "margin-top", 2, "margin-bottom", 2, NULL);
+	gtk_grid_attach (top_grid, widget, 0, 0, 1, 1);
 	dform->url_label = widget;
 
+	// Mirrors - entry
+	widget = gtk_entry_new ();
+	gtk_entry_set_width_chars (GTK_ENTRY (widget), 20);
+	gtk_entry_set_activates_default (GTK_ENTRY (widget), TRUE);
+	g_object_set (widget, "margin-left", 1, "margin-right", 1, NULL);
+	g_object_set (widget, "margin-top", 2, "margin-bottom", 2, NULL);
+	g_object_set (widget, "hexpand", TRUE, NULL);
+	gtk_grid_attach (top_grid, widget, 1, 1, 2, 1);
+	g_signal_connect (GTK_EDITABLE (widget), "changed",
+			G_CALLBACK (on_url_entry_changed), dform);
+	dform->mirrors_entry = widget;
+	// Mirrors - label
+	widget = gtk_label_new_with_mnemonic (_("Mirrors:"));
+	gtk_label_set_mnemonic_widget (GTK_LABEL(widget), dform->mirrors_entry);
+	g_object_set (widget, "margin-left", 3, "margin-right", 3, NULL);
+	g_object_set (widget, "margin-top", 2, "margin-bottom", 2, NULL);
+	gtk_grid_attach (top_grid, widget, 0, 1, 1, 1);
+	dform->mirrors_label = widget;
+
 	// File - entry
 	widget = gtk_entry_new ();
 	gtk_entry_set_activates_default (GTK_ENTRY (widget), TRUE);
-	gtk_table_attach (top_table, widget,  1, 3, 1, 2,
-			GTK_FILL | GTK_EXPAND, GTK_SHRINK, 1, 1);
+	g_object_set (widget, "margin", 1, "hexpand", TRUE, NULL);
+	gtk_grid_attach (top_grid, widget,  1, 2, 2, 1);
 	g_signal_connect (GTK_EDITABLE (widget), "changed",
 			G_CALLBACK (on_entry_changed), dform);
 	dform->file_entry = widget;
 	// File - label
 	widget = gtk_label_new_with_mnemonic (_("File:"));
 	gtk_label_set_mnemonic_widget (GTK_LABEL (widget), dform->file_entry);
-	gtk_table_attach (top_table, widget,  0, 1, 1, 2,
-			GTK_SHRINK, GTK_SHRINK, 3, 1);
+	g_object_set (widget, "margin-left", 3, "margin-right", 3, NULL);
+	g_object_set (widget, "margin-top", 1, "margin-bottom", 1, NULL);
+	gtk_grid_attach (top_grid, widget,  0, 2, 1, 1);
 	dform->file_label = widget;
 
 	// Folder - combo entry + icon
@@ -163,8 +179,8 @@ static void	ug_download_form_init_page1 (UgDownloadForm* dform, UgProxyForm* pro
 			GTK_ENTRY_ICON_SECONDARY, GTK_STOCK_DIRECTORY);
 	gtk_entry_set_icon_tooltip_text (GTK_ENTRY (widget),
 			GTK_ENTRY_ICON_SECONDARY, _("Select Folder"));
-	gtk_table_attach (top_table, dform->folder_combo,  1, 2, 2, 3,
-			GTK_FILL | GTK_EXPAND, GTK_SHRINK, 1, 1);
+	g_object_set (dform->folder_combo, "margin", 1, "hexpand", TRUE, NULL);
+	gtk_grid_attach (top_grid, dform->folder_combo,  1, 3, 1, 1);
 	g_signal_connect (widget, "icon-release",
 			G_CALLBACK (on_select_folder), dform);
 	g_signal_connect (GTK_EDITABLE (widget), "changed",
@@ -172,35 +188,36 @@ static void	ug_download_form_init_page1 (UgDownloadForm* dform, UgProxyForm* pro
 	// Folder - label
 	widget = gtk_label_new_with_mnemonic (_("_Folder:"));
 	gtk_label_set_mnemonic_widget(GTK_LABEL (widget), dform->folder_combo);
-	gtk_table_attach (top_table, widget,  0, 1, 2, 3,
-			GTK_SHRINK, GTK_SHRINK, 3, 1);
+	g_object_set (widget, "margin-left", 3, "margin-right", 3, NULL);
+	g_object_set (widget, "margin-top", 1, "margin-bottom", 1, NULL);
+	gtk_grid_attach (top_grid, widget,  0, 3, 1, 1);
 
 	// Referrer - entry
 	widget = gtk_entry_new ();
 	gtk_entry_set_activates_default (GTK_ENTRY (widget), TRUE);
-	gtk_table_attach (top_table, widget, 1, 3, 3, 4,
-			GTK_FILL | GTK_EXPAND, GTK_SHRINK, 1, 1);
-	gtk_entry_set_activates_default (GTK_ENTRY (widget), TRUE);
+	g_object_set (widget, "margin", 1, "hexpand", TRUE, NULL);
+	gtk_grid_attach (top_grid, widget, 1, 4, 2, 1);
 	g_signal_connect (GTK_EDITABLE (widget), "changed",
 			G_CALLBACK (on_http_entry_changed), dform);
 	dform->referrer_entry = widget;
 	// Referrer - label
 	widget = gtk_label_new_with_mnemonic (_("Referrer:"));
 	gtk_label_set_mnemonic_widget (GTK_LABEL (widget), dform->referrer_entry);
-	gtk_table_attach (top_table, widget, 0, 1, 3, 4,
-			GTK_SHRINK, GTK_SHRINK, 3, 1);
+	g_object_set (widget, "margin-left", 3, "margin-right", 3, NULL);
+	g_object_set (widget, "margin-top", 1, "margin-bottom", 1, NULL);
+	gtk_grid_attach (top_grid, widget, 0, 4, 1, 1);
 //	dform->referrer_label = widget;
 
 	// ----------------------------------------------------
 	// HBox for "Status" and "Login"
-	hbox = gtk_hbox_new (FALSE, 2);
-	gtk_table_attach (top_table, hbox, 0, 3, 4, 5,
-			GTK_FILL | GTK_EXPAND, GTK_SHRINK, 1, 1);
+	hbox = gtk_box_new (GTK_ORIENTATION_HORIZONTAL, 2);
+	g_object_set (hbox, "margin", 1, "hexpand", TRUE, NULL);
+	gtk_grid_attach (top_grid, hbox, 0, 5, 3, 1);
 
 	// ----------------------------------------------------
 	// frame for Status (start mode)
 	frame = gtk_frame_new (_("Status"));
-	vbox = gtk_vbox_new (FALSE, 2);
+	vbox = gtk_box_new (GTK_ORIENTATION_VERTICAL, 2);
 	gtk_container_set_border_width (GTK_CONTAINER (vbox), 2);
 	gtk_container_add (GTK_CONTAINER (frame), vbox);
 	gtk_box_pack_start (GTK_BOX (hbox), frame, FALSE, FALSE, 0);
@@ -214,124 +231,142 @@ static void	ug_download_form_init_page1 (UgDownloadForm* dform, UgProxyForm* pro
 	// ----------------------------------------------------
 	// frame for login
 	frame = gtk_frame_new (_("Login"));
-	table = (GtkTable*) gtk_table_new (2, 2, FALSE);
-	gtk_container_set_border_width (GTK_CONTAINER (table), 2);
-	gtk_container_add (GTK_CONTAINER (frame), (GtkWidget*) table);
+	grid  = (GtkGrid*) gtk_grid_new ();
+	gtk_container_set_border_width (GTK_CONTAINER (grid), 2);
+	gtk_container_add (GTK_CONTAINER (frame), (GtkWidget*) grid);
 	gtk_box_pack_start (GTK_BOX (hbox), frame, TRUE, TRUE, 2);
 	// User - entry
 	widget = gtk_entry_new ();
 	gtk_entry_set_activates_default (GTK_ENTRY (widget), TRUE);
-	gtk_table_attach (table, widget, 1, 2, 0, 1,
-			GTK_FILL | GTK_EXPAND, GTK_SHRINK, 1, 1);
+	g_object_set (widget, "margin", 1, "hexpand", TRUE, NULL);
+	gtk_grid_attach (grid, widget, 1, 0, 1, 1);
 	g_signal_connect (GTK_EDITABLE (widget), "changed",
 			G_CALLBACK (on_entry_changed), dform);
 	dform->username_entry = widget;
 	// User - label
 	widget = gtk_label_new (_("User:"));
-	gtk_table_attach (table, widget, 0, 1, 0, 1,
-			GTK_FILL, GTK_SHRINK, 2, 1);
+	g_object_set (widget, "margin-left", 2, "margin-right", 2, NULL);
+	g_object_set (widget, "margin-top", 1, "margin-bottom", 1, NULL);
+	gtk_grid_attach (grid, widget, 0, 0, 1, 1);
 //	dform->username_label = widget;
 
 	// Password - entry
 	widget = gtk_entry_new ();
 	gtk_entry_set_visibility (GTK_ENTRY (widget), FALSE);
 	gtk_entry_set_activates_default (GTK_ENTRY (widget), TRUE);
-	gtk_table_attach (table, widget, 1, 2, 1, 2,
-			GTK_FILL | GTK_EXPAND, GTK_SHRINK, 1, 1);
+	g_object_set (widget, "margin", 1, "hexpand", TRUE, NULL);
+	gtk_grid_attach (grid, widget, 1, 1, 1, 1);
 	g_signal_connect (GTK_EDITABLE (widget), "changed",
 			G_CALLBACK (on_entry_changed), dform);
 	dform->password_entry = widget;
 	// Password - label
 	widget = gtk_label_new (_("Password:"));
-	gtk_table_attach (table, widget, 0, 1, 1, 2,
-			GTK_FILL, GTK_SHRINK, 2, 1);
+	g_object_set (widget, "margin-left", 2, "margin-right", 2, NULL);
+	g_object_set (widget, "margin-top", 1, "margin-bottom", 1, NULL);
+	gtk_grid_attach (grid, widget, 0, 1, 1, 1);
 //	dform->password_label = widget;
 
 	// ----------------------------------------------------
 	// frame for option
 	frame = gtk_frame_new (_("Options"));
-	table = (GtkTable*) gtk_table_new (2, 7, FALSE);
-	gtk_container_add (GTK_CONTAINER (frame), (GtkWidget*) table);
-	gtk_container_set_border_width (GTK_CONTAINER (table), 2);
-	gtk_table_attach (top_table, frame, 0, 3, 5, 6,
-			GTK_FILL | GTK_EXPAND, GTK_SHRINK, 1, 1);
+	grid  = (GtkGrid*) gtk_grid_new ();
+	gtk_container_add (GTK_CONTAINER (frame), (GtkWidget*) grid);
+	gtk_container_set_border_width (GTK_CONTAINER (grid), 2);
+	g_object_set (frame, "margin", 1, "hexpand", TRUE, NULL);
+	gtk_grid_attach (top_grid, frame, 0, 6, 3, 1);
 	// Retry limit - spin button
-	dform->spin_retry = gtk_spin_button_new_with_range (0.0, 99.0, 1.0);
-	gtk_table_attach (table, dform->spin_retry, 1, 2, 0, 1,
-			GTK_SHRINK, GTK_SHRINK, 2, 1);
-	g_signal_connect (GTK_EDITABLE (dform->spin_retry), "changed",
+	widget = gtk_spin_button_new_with_range (0.0, 99.0, 1.0);
+	g_object_set (widget, "margin-left", 2, "margin-right", 2, NULL);
+	g_object_set (widget, "margin-top", 1, "margin-bottom", 1, NULL);
+	gtk_grid_attach (grid, widget, 1, 0, 1, 1);
+	g_signal_connect (GTK_EDITABLE (widget), "changed",
 			G_CALLBACK (on_spin_changed), dform);
+	dform->spin_retry = widget;
 	// Retry limit - label
 	widget = gtk_label_new_with_mnemonic (_("Retry _limit:"));
 	gtk_label_set_mnemonic_widget (GTK_LABEL (widget), dform->spin_retry);
-	gtk_table_attach (table, widget, 0, 1, 0, 1,
-			GTK_SHRINK, GTK_SHRINK, 2, 1);
+	g_object_set (widget, "margin-left", 2, "margin-right", 2, NULL);
+	g_object_set (widget, "margin-top", 1, "margin-bottom", 1, NULL);
+	gtk_grid_attach (grid, widget, 0, 0, 1, 1);
 	// counts - label
 	widget = gtk_label_new (_("counts"));
 	gtk_misc_set_alignment (GTK_MISC (widget), 0.0, 0.5);
-	gtk_table_attach (table, widget, 2, 3, 0, 1,
-			GTK_FILL, GTK_SHRINK, 2, 1);
+	g_object_set (widget, "margin-left", 2, "margin-right", 2, NULL);
+	g_object_set (widget, "margin-top", 1, "margin-bottom", 1, NULL);
+	gtk_grid_attach (grid, widget, 2, 0, 1, 1);
 	// Retry delay - spin button
-	dform->spin_delay = gtk_spin_button_new_with_range (0.0, 600.0, 1.0);
-	gtk_table_attach (table, dform->spin_delay, 1, 2, 1, 2,
-			GTK_SHRINK, GTK_SHRINK, 2, 1);
-	g_signal_connect (GTK_EDITABLE (dform->spin_delay), "changed",
+	widget = gtk_spin_button_new_with_range (0.0, 600.0, 1.0);
+	g_object_set (widget, "margin-left", 2, "margin-right", 2, NULL);
+	g_object_set (widget, "margin-top", 1, "margin-bottom", 1, NULL);
+	gtk_grid_attach (grid, widget, 1, 1, 1, 1);
+	g_signal_connect (GTK_EDITABLE (widget), "changed",
 			G_CALLBACK (on_spin_changed), dform);
+	dform->spin_delay = widget;
 	// Retry delay - label
 	widget = gtk_label_new_with_mnemonic (_("Retry _delay:"));
 	gtk_label_set_mnemonic_widget (GTK_LABEL (widget), dform->spin_delay);
-	gtk_table_attach (table, widget, 0, 1, 1, 2,
-			GTK_SHRINK, GTK_SHRINK, 2, 1);
+	g_object_set (widget, "margin-left", 2, "margin-right", 2, NULL);
+	g_object_set (widget, "margin-top", 1, "margin-bottom", 1, NULL);
+	gtk_grid_attach (grid, widget, 0, 1, 1, 1);
 	// seconds - label
 	widget = gtk_label_new (_("seconds"));
 	gtk_misc_set_alignment (GTK_MISC (widget), 0.0, 0.5);
-	gtk_table_attach (table, widget, 2, 3, 1, 2,
-			GTK_FILL, GTK_SHRINK, 2, 1);
+	g_object_set (widget, "margin-left", 2, "margin-right", 2, NULL);
+	g_object_set (widget, "margin-top", 1, "margin-bottom", 1, NULL);
+	gtk_grid_attach (grid, widget, 2, 1, 1, 1);
 
 	// Connections per server
 	// separator
-	gtk_table_attach (table, gtk_vseparator_new (), 3, 4, 0, 2,
-			GTK_FILL | GTK_EXPAND, GTK_FILL, 2, 1);
+	widget = gtk_separator_new (GTK_ORIENTATION_VERTICAL);
+	g_object_set (widget, "margin-left", 2, "margin-right", 2, NULL);
+	g_object_set (widget, "margin-top", 1, "margin-bottom", 1, NULL);
+	gtk_grid_attach (grid, widget, 3, 0, 1, 2);
 	// "Connections per server" - title label
 	widget = gtk_label_new (_("Connections per server"));
-	gtk_table_attach (table, widget, 4, 7, 0, 1,
-			GTK_SHRINK, GTK_SHRINK, 2, 1);
+	g_object_set (widget, "margin-left", 2, "margin-right", 2, NULL);
+	g_object_set (widget, "margin-top", 1, "margin-bottom", 1, NULL);
+	gtk_grid_attach (grid, widget, 4, 0, 3, 1);
 	dform->title_connections = widget;
 	// connections - spin button
 	widget = gtk_spin_button_new_with_range (1.0, 20.0, 1.0);
 	gtk_entry_set_width_chars (GTK_ENTRY (widget), 3);
-	gtk_table_attach (table, widget, 5, 6, 1, 2,
-			GTK_SHRINK, GTK_SHRINK, 2, 1);
+	g_object_set (widget, "margin-left", 2, "margin-right", 2, NULL);
+	g_object_set (widget, "margin-top", 1, "margin-bottom", 1, NULL);
+	gtk_grid_attach (grid, widget, 5, 1, 1, 1);
 	dform->spin_connections = widget;
 	// connections - label
 	widget = gtk_label_new (_("connections"));
 	gtk_misc_set_alignment (GTK_MISC (widget), 0.0, 0.5);
-	gtk_table_attach (table, widget, 6, 7, 1, 2,
-			GTK_FILL, GTK_SHRINK, 2, 1);
+	g_object_set (widget, "margin-left", 2, "margin-right", 2, NULL);
+	g_object_set (widget, "margin-top", 1, "margin-bottom", 1, NULL);
+	gtk_grid_attach (grid, widget, 6, 1, 1, 1);
 	dform->label_connections = widget;
 
 	// ----------------------------------------------------
 	// proxy
-//	ug_proxy_widget_init (&dform->proxy_dform, TRUE);
-	if (proxy)
-		gtk_table_attach (top_table, proxy->self, 0, 3, 6, 7,
-				GTK_FILL | GTK_EXPAND, GTK_SHRINK, 1, 1);
+//	ug_proxy_widget_init (&dform->proxy_dform);
+	if (proxy) {
+		widget = proxy->self;
+		g_object_set (widget, "margin", 1, "hexpand", TRUE, NULL);
+		gtk_grid_attach (top_grid, widget, 0, 7, 3, 1);
+	}
 }
 
 static void	ug_download_form_init_page2 (UgDownloadForm* dform)
 {
 	GtkWidget*	widget;
-	GtkTable*	table;
+	GtkGrid*	grid;
 
-	dform->page2 = gtk_table_new (7, 4, FALSE);
-	table = (GtkTable*) dform->page2;
-	gtk_container_set_border_width (GTK_CONTAINER (table), 2);
+	dform->page2 = gtk_grid_new ();
+	grid = (GtkGrid*) dform->page2;
+	gtk_container_set_border_width (GTK_CONTAINER (grid), 2);
 
 	// label - cookie file
 	widget = gtk_label_new (_("Cookie file:"));
 	gtk_misc_set_alignment (GTK_MISC (widget), 0.0, 0.5);	// left, center
-	gtk_table_attach (table, widget, 0, 1, 0, 1,
-			GTK_SHRINK, GTK_SHRINK, 2, 1);
+	g_object_set (widget, "margin-left", 2, "margin-right", 2, NULL);
+	g_object_set (widget, "margin-top", 1, "margin-bottom", 1, NULL);
+	gtk_grid_attach (grid, widget, 0, 0, 1, 1);
 	dform->cookie_label = widget;
 	// entry - cookie file
 	widget = gtk_entry_new ();
@@ -340,8 +375,8 @@ static void	ug_download_form_init_page2 (UgDownloadForm* dform)
 			GTK_ENTRY_ICON_SECONDARY, GTK_STOCK_FILE);
 	gtk_entry_set_icon_tooltip_text (GTK_ENTRY (widget),
 			GTK_ENTRY_ICON_SECONDARY, _("Select Cookie File"));
-	gtk_table_attach (table, widget, 1, 4, 0, 1,
-			GTK_FILL | GTK_EXPAND, GTK_SHRINK, 1, 1);
+	g_object_set (widget, "margin", 1, "hexpand", TRUE, NULL);
+	gtk_grid_attach (grid, widget, 1, 0, 3, 1);
 	g_signal_connect (widget, "icon-release",
 			G_CALLBACK (on_select_cookie), dform);
 	g_signal_connect (GTK_EDITABLE (widget), "changed",
@@ -350,8 +385,9 @@ static void	ug_download_form_init_page2 (UgDownloadForm* dform)
 	// label - post file
 	widget = gtk_label_new (_("Post file:"));
 	gtk_misc_set_alignment (GTK_MISC (widget), 0.0, 0.5);	// left, center
-	gtk_table_attach (table, widget, 0, 1, 1, 2,
-			GTK_SHRINK, GTK_SHRINK, 2, 1);
+	g_object_set (widget, "margin-left", 2, "margin-right", 2, NULL);
+	g_object_set (widget, "margin-top", 1, "margin-bottom", 1, NULL);
+	gtk_grid_attach (grid, widget, 0, 1, 1, 1);
 	dform->post_label = widget;
 	// entry - post file
 	widget = gtk_entry_new ();
@@ -360,45 +396,68 @@ static void	ug_download_form_init_page2 (UgDownloadForm* dform)
 			GTK_ENTRY_ICON_SECONDARY, GTK_STOCK_FILE);
 	gtk_entry_set_icon_tooltip_text (GTK_ENTRY (widget),
 			GTK_ENTRY_ICON_SECONDARY, _("Select Post File"));
-	gtk_table_attach (table, widget, 1, 4, 1, 2,
-			GTK_FILL | GTK_EXPAND, GTK_SHRINK, 1, 1);
+	g_object_set (widget, "margin", 1, "hexpand", TRUE, NULL);
+	gtk_grid_attach (grid, widget, 1, 1, 3, 1);
 	g_signal_connect (widget, "icon-release",
 			G_CALLBACK (on_select_post), dform);
 	g_signal_connect (GTK_EDITABLE (widget), "changed",
 			G_CALLBACK (on_http_entry_changed), dform);
 	dform->post_entry = widget;
 
+	// label - user agent
+	widget = gtk_label_new (_("User Agent:"));
+	gtk_misc_set_alignment (GTK_MISC (widget), 0.0, 0.5);	// left, center
+	g_object_set (widget, "margin-left", 2, "margin-right", 2, NULL);
+	g_object_set (widget, "margin-top", 1, "margin-bottom", 1, NULL);
+	gtk_grid_attach (grid, widget, 0, 2, 1, 1);
+	dform->agent_label = widget;
+	// entry - user agent
+	widget = gtk_entry_new ();
+	gtk_entry_set_activates_default (GTK_ENTRY (widget), TRUE);
+	g_object_set (widget, "margin", 1, "hexpand", TRUE, NULL);
+	gtk_grid_attach (grid, widget, 1, 2, 3, 1);
+	g_signal_connect (GTK_EDITABLE (widget), "changed",
+			G_CALLBACK (on_http_entry_changed), dform);
+	dform->agent_entry = widget;
+
 	// label - Max upload speed
 	widget = gtk_label_new (_("Max upload speed:"));
-	gtk_table_attach (table, widget, 0, 2, 2, 3,
-			GTK_SHRINK, GTK_SHRINK, 2, 1);
+	g_object_set (widget, "margin-left", 2, "margin-right", 2, NULL);
+	g_object_set (widget, "margin-top", 1, "margin-bottom", 1, NULL);
+	gtk_grid_attach (grid, widget, 0, 3, 2, 1);
 	// spin - Max upload speed
 	widget = gtk_spin_button_new_with_range (0, 99999999, 1);
 	gtk_entry_set_width_chars (GTK_ENTRY (widget), 8);
-	gtk_table_attach (table, widget, 2, 3, 2, 3,
-			GTK_FILL, GTK_SHRINK, 1, 1);
+	g_object_set (widget, "margin", 1, NULL);
+	gtk_grid_attach (grid, widget, 2, 3, 1, 1);
 	dform->spin_upload_speed = (GtkSpinButton*) widget;
 	// label - "KiB/s"
-	widget = gtk_label_new ("KiB/s");
+	widget = gtk_label_new (_("KiB/s"));
 	gtk_misc_set_alignment (GTK_MISC (widget), 0.0, 0.5);	// left, center
-	gtk_table_attach (table, widget, 3, 4, 2, 3,
-			GTK_FILL | GTK_EXPAND, GTK_SHRINK, 2, 2);
+	g_object_set (widget, "margin", 2, "hexpand", TRUE, NULL);
+	gtk_grid_attach (grid, widget, 3, 3, 1, 1);
 
 	// label - Max download speed
 	widget = gtk_label_new (_("Max download speed:"));
-	gtk_table_attach (table, widget, 0, 2, 3, 4,
-			GTK_SHRINK, GTK_SHRINK, 2, 1);
+	g_object_set (widget, "margin-left", 2, "margin-right", 2, NULL);
+	g_object_set (widget, "margin-top", 1, "margin-bottom", 1, NULL);
+	gtk_grid_attach (grid, widget, 0, 4, 2, 1);
 	// spin - Max download speed
 	widget = gtk_spin_button_new_with_range (0, 99999999, 1);
 	gtk_entry_set_width_chars (GTK_ENTRY (widget), 8);
-	gtk_table_attach (table, widget, 2, 3, 3, 4,
-			GTK_FILL, GTK_SHRINK, 1, 1);
+	g_object_set (widget, "margin", 1, NULL);
+	gtk_grid_attach (grid, widget, 2, 4, 1, 1);
 	dform->spin_download_speed = (GtkSpinButton*) widget;
 	// label - "KiB/s"
-	widget = gtk_label_new ("KiB/s");
+	widget = gtk_label_new (_("KiB/s"));
 	gtk_misc_set_alignment (GTK_MISC (widget), 0.0, 0.5);	// left, center
-	gtk_table_attach (table, widget, 3, 4, 3, 4,
-			GTK_FILL | GTK_EXPAND, GTK_SHRINK, 2, 2);
+	g_object_set (widget, "margin", 2, "hexpand", TRUE, NULL);
+	gtk_grid_attach (grid, widget, 3, 4, 1, 1);
+
+	// Retrieve timestamp
+	widget = gtk_check_button_new_with_label (_("Retrieve timestamp"));
+	gtk_grid_attach (grid, widget, 0, 5, 3, 1);
+	dform->timestamp = (GtkToggleButton*) widget;
 }
 
 static void ug_download_form_decide_sensitive (UgDownloadForm* dform)
@@ -407,6 +466,8 @@ static void ug_download_form_decide_sensitive (UgDownloadForm* dform)
 	gboolean	sensitive;
 
 	sensitive = app->setting.plugin.aria2.enable;
+	gtk_widget_set_sensitive (dform->mirrors_label, sensitive);
+	gtk_widget_set_sensitive (dform->mirrors_entry, sensitive);
 	gtk_widget_set_sensitive (dform->title_connections, sensitive);
 	gtk_widget_set_sensitive (dform->label_connections, sensitive);
 	gtk_widget_set_sensitive (dform->spin_connections, sensitive);
@@ -455,9 +516,12 @@ void	ug_download_form_get  (UgDownloadForm* dform, UgDataset* dataset)
 	// max_connections
 	number = gtk_spin_button_get_value_as_int ((GtkSpinButton*) dform->spin_connections);
 	common->max_connections = number;
+	// timestamp
+	common->retrieve_timestamp = gtk_toggle_button_get_active (dform->timestamp);
 
 	if (gtk_widget_is_sensitive (dform->url_entry) == TRUE) {
-		ug_str_set (&common->url,  gtk_entry_get_text ((GtkEntry*)dform->url_entry),  -1);
+		ug_str_set (&common->url, gtk_entry_get_text ((GtkEntry*)dform->url_entry), -1);
+		ug_str_set (&common->mirrors, gtk_entry_get_text ((GtkEntry*)dform->mirrors_entry), -1);
 		ug_str_set (&common->file, gtk_entry_get_text ((GtkEntry*)dform->file_entry), -1);
 		if (common->url) {
 			ug_uri_full_init (&urifull, common->url);
@@ -493,6 +557,9 @@ void	ug_download_form_get  (UgDownloadForm* dform, UgDataset* dataset)
 	// post_file
 	text = gtk_entry_get_text ((GtkEntry*) dform->post_entry);
 	ug_str_set (&http->post_file, text, -1);
+	// user_agent
+	text = gtk_entry_get_text ((GtkEntry*) dform->agent_entry);
+	ug_str_set (&http->user_agent, text, -1);
 
 	// ------------------------------------------
 	// UgRelation
@@ -521,6 +588,7 @@ void	ug_download_form_set (UgDownloadForm* dform, UgDataset* dataset, gboolean k
 	// set changed flags
 	if (keep_changed==FALSE && common) {
 		dform->changed.url      = common->keeping.url;
+		dform->changed.mirrors  = common->keeping.mirrors;
 		dform->changed.file     = common->keeping.file;
 		dform->changed.folder   = common->keeping.folder;
 		dform->changed.user     = common->keeping.user;
@@ -534,7 +602,13 @@ void	ug_download_form_set (UgDownloadForm* dform, UgDataset* dataset, gboolean k
 	if (keep_changed==FALSE || dform->changed.url==FALSE) {
 		if (gtk_widget_is_sensitive (dform->url_entry)) {
 			gtk_entry_set_text ((GtkEntry*) dform->url_entry,
-					(common && common->url)  ? common->url  : "");
+					(common && common->url)  ? common->url : "");
+		}
+	}
+	if (keep_changed==FALSE || dform->changed.mirrors==FALSE) {
+		if (gtk_widget_is_sensitive (dform->mirrors_entry)) {
+			gtk_entry_set_text ((GtkEntry*) dform->mirrors_entry,
+					(common && common->mirrors)  ? common->mirrors : "");
 		}
 	}
 	if (keep_changed==FALSE || dform->changed.file==FALSE) {
@@ -582,6 +656,8 @@ void	ug_download_form_set (UgDownloadForm* dform, UgDataset* dataset, gboolean k
 		gtk_spin_button_set_value ((GtkSpinButton*) dform->spin_connections,
 			common->max_connections);
 	}
+	if (keep_changed==FALSE || dform->changed.timestamp==FALSE)
+		gtk_toggle_button_set_active (dform->timestamp, common->retrieve_timestamp);
 
 	// ------------------------------------------
 	// UgDataHttp
@@ -598,11 +674,16 @@ void	ug_download_form_set (UgDownloadForm* dform, UgDataset* dataset, gboolean k
 		gtk_entry_set_text ((GtkEntry*) dform->post_entry,
 				(http && http->post_file) ? http->post_file : "");
 	}
+	if (keep_changed==FALSE || dform->changed.agent==FALSE) {
+		gtk_entry_set_text ((GtkEntry*) dform->agent_entry,
+				(http && http->user_agent) ? http->user_agent : "");
+	}
 	// set changed flags
 	if (keep_changed==FALSE && http) {
 		dform->changed.referrer = http->keeping.referrer;
 		dform->changed.cookie   = http->keeping.cookie_file;
 		dform->changed.post     = http->keeping.post_file;
+		dform->changed.agent    = http->keeping.user_agent;
 	}
 
 	// ------------------------------------------
@@ -628,12 +709,16 @@ void	ug_download_form_set_multiple (UgDownloadForm* dform, gboolean multiple_mod
 	if (multiple_mode) {
 		gtk_widget_hide (dform->url_label);
 		gtk_widget_hide (dform->url_entry);
+		gtk_widget_hide (dform->mirrors_label);
+		gtk_widget_hide (dform->mirrors_entry);
 		gtk_widget_hide (dform->file_label);
 		gtk_widget_hide (dform->file_entry);
 	}
 	else {
 		gtk_widget_show (dform->url_label);
 		gtk_widget_show (dform->url_entry);
+		gtk_widget_show (dform->mirrors_label);
+		gtk_widget_show (dform->mirrors_entry);
 		gtk_widget_show (dform->file_label);
 		gtk_widget_show (dform->file_entry);
 	}
@@ -670,12 +755,10 @@ void	ug_download_form_set_folder_list (UgDownloadForm* dform, GList* folder_list
 
 void	ug_download_form_get_folder_list (UgDownloadForm* dform, GList** folder_list)
 {
-	GtkComboBox*	combo;
 	const gchar*	current;
 	GList*			link;
 	guint			length;
 
-	combo   = GTK_COMBO_BOX (dform->folder_combo);
 	current = gtk_entry_get_text ((GtkEntry*) dform->folder_entry);
 	if (*current == 0)
 		return;
@@ -703,7 +786,7 @@ void	ug_download_form_complete_entry (UgDownloadForm* dform)
 {
 	UgUriFull		urifull;
 	const gchar*	text;
-	gchar*			temp;
+//	gchar*			temp;
 	gboolean		completed;
 
 	// URL
@@ -712,6 +795,7 @@ void	ug_download_form_complete_entry (UgDownloadForm* dform)
 	if (urifull.host) {
 		// disable changed flags
 		dform->changed.enable = FALSE;
+/*
 		// complete file entry
 		text = gtk_entry_get_text ((GtkEntry*) dform->file_entry);
 		if (text[0] == 0 || dform->changed.file == FALSE) {
@@ -720,7 +804,6 @@ void	ug_download_form_complete_entry (UgDownloadForm* dform)
 					(temp) ? temp : "index.htm");
 			g_free (temp);
 		}
-/*
 		// complete user entry
 		text = gtk_entry_get_text ((GtkEntry*) dform->username_entry);
 		if (text[0] == 0 || dform->changed.user == FALSE) {
@@ -795,6 +878,8 @@ static void	on_http_entry_changed (GtkEditable* editable, UgDownloadForm* dform)
 			dform->changed.cookie = TRUE;
 		else if (editable == GTK_EDITABLE (dform->post_entry))
 			dform->changed.post = TRUE;
+		else if (editable == GTK_EDITABLE (dform->agent_entry))
+			dform->changed.agent = TRUE;
 	}
 }
 
