@@ -1,6 +1,6 @@
 /*
  *
- *   Copyright (C) 2005-2012 by C.H. Huang
+ *   Copyright (C) 2005-2013 by C.H. Huang
  *   plushuang.tw@gmail.com
  *
  *  This library is free software; you can redistribute it and/or
@@ -34,6 +34,7 @@
  *
  */
 
+#include <UgUri.h>
 #include <UgUtils.h>
 #include <UgString.h>
 #include <UgData-download.h>
@@ -294,6 +295,10 @@ static void col_set_icon (GtkTreeViewColumn *tree_column,
 	const gchar*	stock_id;
 
 	gtk_tree_model_get (model, iter, 0, &dataset, -1);
+	// avoid crash in GTK3
+	if (dataset == NULL)
+		return;
+
 	relation = UG_DATASET_RELATION (dataset);
 	// set stock_id
 	if (relation->hints & UG_HINT_DOWNLOADING)
@@ -321,6 +326,10 @@ static void col_set_icon_all (GtkTreeViewColumn *tree_column,
 	const gchar*	stock_id;
 
 	gtk_tree_model_get (model, iter, 0, &dataset, -1);
+	// avoid crash in GTK3
+	if (dataset == NULL)
+		return;
+
 	relation = UG_DATASET_RELATION (dataset);
 	// set stock_id
 	if (relation->hints & UG_HINT_RECYCLED)
@@ -350,18 +359,28 @@ static void col_set_name (GtkTreeViewColumn *tree_column,
 	UgDataset*		dataset;
 	UgDataCommon*	common;
 	gchar*			string = NULL;
+	gchar*			name = NULL;
 
 	gtk_tree_model_get (model, iter, 0, &dataset, -1);
-	common = UG_DATASET_COMMON (dataset);
+	// avoid crash in GTK3
+	if (dataset == NULL)
+		return;
 
+	common = UG_DATASET_COMMON (dataset);
 	if (common->name)
 		string = common->name;
 	else if (common->file)
 		string = common->file;
-	else
-		string = _("unnamed");
+	else {
+		if (common->url)
+			name = ug_uri_get_filename (common->url);
+		if (name == NULL)
+			name = g_strconcat (_("unnamed"), " ", common->url, NULL);
+		string = name;
+	}
 
 	g_object_set (cell, "text", string, NULL);
+	g_free (name);
 }
 
 static void col_set_complete (GtkTreeViewColumn *tree_column,
@@ -375,8 +394,11 @@ static void col_set_complete (GtkTreeViewColumn *tree_column,
 	gchar*			string;
 
 	gtk_tree_model_get (model, iter, 0, &dataset, -1);
-	progress = UG_DATASET_PROGRESS (dataset);
+	// avoid crash in GTK3
+	if (dataset == NULL)
+		return;
 
+	progress = UG_DATASET_PROGRESS (dataset);
 	if (progress)
 		string = ug_str_dtoa_unit ((gdouble) progress->complete, 1, NULL);
 	else
@@ -397,8 +419,11 @@ static void col_set_total (GtkTreeViewColumn *tree_column,
 	gchar*		string;
 
 	gtk_tree_model_get (model, iter, 0, &dataset, -1);
-	progress = UG_DATASET_PROGRESS (dataset);
+	// avoid crash in GTK3
+	if (dataset == NULL)
+		return;
 
+	progress = UG_DATASET_PROGRESS (dataset);
 	if (progress)
 		string = ug_str_dtoa_unit ((gdouble) progress->total, 1, NULL);
 	else
@@ -419,8 +444,11 @@ static void col_set_percent (GtkTreeViewColumn *tree_column,
 	gchar*		string;
 
 	gtk_tree_model_get (model, iter, 0, &dataset, -1);
-	progress = UG_DATASET_PROGRESS (dataset);
+	// avoid crash in GTK3
+	if (dataset == NULL)
+		return;
 
+	progress = UG_DATASET_PROGRESS (dataset);
 	if (progress) {
 //		string = g_strdup_printf ("%.*f%c", 1, progress->percent, '%');
 		string = g_strdup_printf ("%.1f%c", progress->percent, '%');
@@ -447,8 +475,11 @@ static void col_set_consume_time (GtkTreeViewColumn *tree_column,
 	gchar*		string;
 
 	gtk_tree_model_get (model, iter, 0, &dataset, -1);
-	progress = UG_DATASET_PROGRESS (dataset);
+	// avoid crash in GTK3
+	if (dataset == NULL)
+		return;
 
+	progress = UG_DATASET_PROGRESS (dataset);
 	if (progress)
 		string = ug_str_from_seconds ((guint) progress->consume_time, TRUE);
 	else
@@ -470,9 +501,12 @@ static void col_set_remain_time (GtkTreeViewColumn *tree_column,
 	gchar*		string;
 
 	gtk_tree_model_get (model, iter, 0, &dataset, -1);
+	// avoid crash in GTK3
+	if (dataset == NULL)
+		return;
+
 	progress = UG_DATASET_PROGRESS (dataset);
 	relation = UG_DATASET_RELATION (dataset);
-
 	if (progress && relation && relation->plugin)
 		string = ug_str_from_seconds ((guint) progress->remain_time, TRUE);
 	else
@@ -494,9 +528,12 @@ static void col_set_speed (GtkTreeViewColumn *tree_column,
 	gchar*		string;
 
 	gtk_tree_model_get (model, iter, 0, &dataset, -1);
+	// avoid crash in GTK3
+	if (dataset == NULL)
+		return;
+
 	progress = UG_DATASET_PROGRESS (dataset);
 	relation = UG_DATASET_RELATION (dataset);
-
 	if (progress && relation && relation->plugin)
 		string = ug_str_dtoa_unit ((gdouble) progress->download_speed, 1, "/s");
 	else
@@ -518,9 +555,12 @@ static void col_set_upload_speed (GtkTreeViewColumn *tree_column,
 	gchar*		string;
 
 	gtk_tree_model_get (model, iter, 0, &dataset, -1);
+	// avoid crash in GTK3
+	if (dataset == NULL)
+		return;
+
 	progress = UG_DATASET_PROGRESS (dataset);
 	relation = UG_DATASET_RELATION (dataset);
-
 	if (progress && relation && relation->plugin && progress->upload_speed)
 		string = ug_str_dtoa_unit ((gdouble) progress->upload_speed, 1, "/s");
 	else
@@ -537,14 +577,15 @@ static void col_set_uploaded (GtkTreeViewColumn *tree_column,
                            gpointer           data)
 {
 	UgDataset*	dataset;
-	UgRelation*	relation;
 	UgProgress*	progress;
 	gchar*		string;
 
 	gtk_tree_model_get (model, iter, 0, &dataset, -1);
-	progress = UG_DATASET_PROGRESS (dataset);
-	relation = UG_DATASET_RELATION (dataset);
+	// avoid crash in GTK3
+	if (dataset == NULL)
+		return;
 
+	progress = UG_DATASET_PROGRESS (dataset);
 	if (progress && progress->uploaded)
 		string = ug_str_dtoa_unit ((gdouble) progress->uploaded, 1, NULL);
 	else
@@ -561,14 +602,15 @@ static void col_set_ratio (GtkTreeViewColumn *tree_column,
                            gpointer           data)
 {
 	UgDataset*	dataset;
-	UgRelation*	relation;
 	UgProgress*	progress;
 	gchar*		string;
 
 	gtk_tree_model_get (model, iter, 0, &dataset, -1);
-	progress = UG_DATASET_PROGRESS (dataset);
-	relation = UG_DATASET_RELATION (dataset);
+	// avoid crash in GTK3
+	if (dataset == NULL)
+		return;
 
+	progress = UG_DATASET_PROGRESS (dataset);
 	if (progress && progress->ratio)
 		string = g_strdup_printf ("%.2f", progress->ratio);
 	else
@@ -589,8 +631,11 @@ static void col_set_retry (GtkTreeViewColumn *tree_column,
 	gchar*			string;
 
 	gtk_tree_model_get (model, iter, 0, &dataset, -1);
-	common = UG_DATASET_COMMON (dataset);
+	// avoid crash in GTK3
+	if (dataset == NULL)
+		return;
 
+	common = UG_DATASET_COMMON (dataset);
 	if (common->retry_count != 0)
 		string = g_strdup_printf ("%d", common->retry_count);
 	else
@@ -611,8 +656,11 @@ static void col_set_category (GtkTreeViewColumn *tree_column,
 	gchar*			string;
 
 	gtk_tree_model_get (model, iter, 0, &dataset, -1);
-	relation = UG_DATASET_RELATION (dataset);
+	// avoid crash in GTK3
+	if (dataset == NULL)
+		return;
 
+	relation = UG_DATASET_RELATION (dataset);
 	if (relation && relation->category)
 		string = relation->category->name;
 	else
@@ -630,6 +678,9 @@ static void col_set_url (GtkTreeViewColumn *tree_column,
 	UgDataset*	dataset;
 
 	gtk_tree_model_get (model, iter, 0, &dataset, -1);
+	// avoid crash in GTK3
+	if (dataset == NULL)
+		return;
 
 	g_object_set (cell, "text", UG_DATASET_COMMON (dataset)->url, NULL);
 }
@@ -645,6 +696,10 @@ static void col_set_added_on (GtkTreeViewColumn *tree_column,
 	gchar*		string;
 
 	gtk_tree_model_get (model, iter, 0, &dataset, -1);
+	// avoid crash in GTK3
+	if (dataset == NULL)
+		return;
+
 	datalog = ug_dataset_get (dataset, UG_DATA_LOG_I, 0);
 	string = (datalog) ? datalog->added_on : NULL;
 
@@ -662,6 +717,10 @@ static void col_set_completed_on (GtkTreeViewColumn *tree_column,
 	gchar*		string;
 
 	gtk_tree_model_get (model, iter, 0, &dataset, -1);
+	// avoid crash in GTK3
+	if (dataset == NULL)
+		return;
+
 	datalog = ug_dataset_get (dataset, UG_DATA_LOG_I, 0);
 	string = (datalog) ? datalog->completed_on : NULL;
 
