@@ -1,6 +1,6 @@
 /*
  *
- *   Copyright (C) 2005-2012 by C.H. Huang
+ *   Copyright (C) 2005-2013 by C.H. Huang
  *   plushuang.tw@gmail.com
  *
  *  This library is free software; you can redistribute it and/or
@@ -52,23 +52,23 @@ static void	ug_schedule_state_to_markup (guint (*state)[7][24], UgMarkup* markup
 
 static const UgDataEntry	ug_download_column_setting_entry[] =
 {
-	{"completed",	G_STRUCT_OFFSET (struct UgDownloadColumnSetting, completed),	UG_DATA_INT,	NULL,	NULL},
-	{"total",		G_STRUCT_OFFSET (struct UgDownloadColumnSetting, total),		UG_DATA_INT,	NULL,	NULL},
-	{"percent",		G_STRUCT_OFFSET (struct UgDownloadColumnSetting, percent),		UG_DATA_INT,	NULL,	NULL},
-	{"elapsed",		G_STRUCT_OFFSET (struct UgDownloadColumnSetting, elapsed),		UG_DATA_INT,	NULL,	NULL},
-	{"left",		G_STRUCT_OFFSET (struct UgDownloadColumnSetting, left),			UG_DATA_INT,	NULL,	NULL},
-	{"speed",		G_STRUCT_OFFSET (struct UgDownloadColumnSetting, speed),		UG_DATA_INT,	NULL,	NULL},
-	{"UploadSpeed",	G_STRUCT_OFFSET (struct UgDownloadColumnSetting, upload_speed),	UG_DATA_INT,	NULL,	NULL},
-	{"uploaded",	G_STRUCT_OFFSET (struct UgDownloadColumnSetting, uploaded),		UG_DATA_INT,	NULL,	NULL},
-	{"ratio",		G_STRUCT_OFFSET (struct UgDownloadColumnSetting, ratio),		UG_DATA_INT,	NULL,	NULL},
-	{"retry",		G_STRUCT_OFFSET (struct UgDownloadColumnSetting, retry),		UG_DATA_INT,	NULL,	NULL},
-	{"category",	G_STRUCT_OFFSET (struct UgDownloadColumnSetting, category),		UG_DATA_INT,	NULL,	NULL},
-	{"URL",			G_STRUCT_OFFSET (struct UgDownloadColumnSetting, url),			UG_DATA_INT,	NULL,	NULL},
-	{"AddedOn",		G_STRUCT_OFFSET (struct UgDownloadColumnSetting, added_on),		UG_DATA_INT,	NULL,	NULL},
-	{"CompletedOn",	G_STRUCT_OFFSET (struct UgDownloadColumnSetting, completed_on),	UG_DATA_INT,	NULL,	NULL},
+	{"completed",	G_STRUCT_OFFSET (struct UgDownloadColumnSetting, completed),	UG_TYPE_INT,	NULL,	NULL},
+	{"total",		G_STRUCT_OFFSET (struct UgDownloadColumnSetting, total),		UG_TYPE_INT,	NULL,	NULL},
+	{"percent",		G_STRUCT_OFFSET (struct UgDownloadColumnSetting, percent),		UG_TYPE_INT,	NULL,	NULL},
+	{"elapsed",		G_STRUCT_OFFSET (struct UgDownloadColumnSetting, elapsed),		UG_TYPE_INT,	NULL,	NULL},
+	{"left",		G_STRUCT_OFFSET (struct UgDownloadColumnSetting, left),			UG_TYPE_INT,	NULL,	NULL},
+	{"speed",		G_STRUCT_OFFSET (struct UgDownloadColumnSetting, speed),		UG_TYPE_INT,	NULL,	NULL},
+	{"UploadSpeed",	G_STRUCT_OFFSET (struct UgDownloadColumnSetting, upload_speed),	UG_TYPE_INT,	NULL,	NULL},
+	{"uploaded",	G_STRUCT_OFFSET (struct UgDownloadColumnSetting, uploaded),		UG_TYPE_INT,	NULL,	NULL},
+	{"ratio",		G_STRUCT_OFFSET (struct UgDownloadColumnSetting, ratio),		UG_TYPE_INT,	NULL,	NULL},
+	{"retry",		G_STRUCT_OFFSET (struct UgDownloadColumnSetting, retry),		UG_TYPE_INT,	NULL,	NULL},
+	{"category",	G_STRUCT_OFFSET (struct UgDownloadColumnSetting, category),		UG_TYPE_INT,	NULL,	NULL},
+	{"URL",			G_STRUCT_OFFSET (struct UgDownloadColumnSetting, url),			UG_TYPE_INT,	NULL,	NULL},
+	{"AddedOn",		G_STRUCT_OFFSET (struct UgDownloadColumnSetting, added_on),		UG_TYPE_INT,	NULL,	NULL},
+	{"CompletedOn",	G_STRUCT_OFFSET (struct UgDownloadColumnSetting, completed_on),	UG_TYPE_INT,	NULL,	NULL},
 
-	{"sort-nth",	G_STRUCT_OFFSET (struct UgDownloadColumnSetting, sort.nth),		UG_DATA_INT,	NULL,	NULL},
-	{"sort-order",	G_STRUCT_OFFSET (struct UgDownloadColumnSetting, sort.order),	UG_DATA_INT,	NULL,	NULL},
+	{"sort-nth",	G_STRUCT_OFFSET (struct UgDownloadColumnSetting, sort.nth),		UG_TYPE_INT,	NULL,	NULL},
+	{"sort-order",	G_STRUCT_OFFSET (struct UgDownloadColumnSetting, sort.order),	UG_TYPE_INT,	NULL,	NULL},
 	{NULL}			// null-terminated
 };
 
@@ -84,11 +84,11 @@ static const UgDataInterface	ug_download_column_setting_iface =
 // UgSummarySetting
 static const UgDataEntry	ug_summary_setting_entry[] =
 {
-	{"name",		G_STRUCT_OFFSET (struct UgSummarySetting, name),			UG_DATA_INT,	NULL,	NULL},
-	{"folder",		G_STRUCT_OFFSET (struct UgSummarySetting, folder),			UG_DATA_INT,	NULL,	NULL},
-	{"category",	G_STRUCT_OFFSET (struct UgSummarySetting, category),		UG_DATA_INT,	NULL,	NULL},
-	{"URL",			G_STRUCT_OFFSET (struct UgSummarySetting, url),				UG_DATA_INT,	NULL,	NULL},
-	{"message",		G_STRUCT_OFFSET (struct UgSummarySetting, message),			UG_DATA_INT,	NULL,	NULL},
+	{"name",		G_STRUCT_OFFSET (struct UgSummarySetting, name),			UG_TYPE_INT,	NULL,	NULL},
+	{"folder",		G_STRUCT_OFFSET (struct UgSummarySetting, folder),			UG_TYPE_INT,	NULL,	NULL},
+	{"category",	G_STRUCT_OFFSET (struct UgSummarySetting, category),		UG_TYPE_INT,	NULL,	NULL},
+	{"URL",			G_STRUCT_OFFSET (struct UgSummarySetting, url),				UG_TYPE_INT,	NULL,	NULL},
+	{"message",		G_STRUCT_OFFSET (struct UgSummarySetting, message),			UG_TYPE_INT,	NULL,	NULL},
 	{NULL}			// null-terminated
 };
 
@@ -104,15 +104,15 @@ static const UgDataInterface	ug_summary_setting_iface =
 // UgWindowSetting
 static const UgDataEntry	ug_window_setting_entry[] =
 {
-	{"Toolbar",		G_STRUCT_OFFSET (struct UgWindowSetting, toolbar),		UG_DATA_INT,	NULL,	NULL},
-	{"Statusbar",	G_STRUCT_OFFSET (struct UgWindowSetting, statusbar),	UG_DATA_INT,	NULL,	NULL},
-	{"Category",	G_STRUCT_OFFSET (struct UgWindowSetting, category),		UG_DATA_INT,	NULL,	NULL},
-	{"Summary",		G_STRUCT_OFFSET (struct UgWindowSetting, summary),		UG_DATA_INT,	NULL,	NULL},
-	{"x",			G_STRUCT_OFFSET (struct UgWindowSetting, x)	,			UG_DATA_INT,	NULL,	NULL},
-	{"y",			G_STRUCT_OFFSET (struct UgWindowSetting, y),			UG_DATA_INT,	NULL,	NULL},
-	{"width",		G_STRUCT_OFFSET (struct UgWindowSetting, width),		UG_DATA_INT,	NULL,	NULL},
-	{"height",		G_STRUCT_OFFSET (struct UgWindowSetting, height),		UG_DATA_INT,	NULL,	NULL},
-	{"maximized",	G_STRUCT_OFFSET (struct UgWindowSetting, maximized),	UG_DATA_INT,	NULL,	NULL},
+	{"Toolbar",		G_STRUCT_OFFSET (struct UgWindowSetting, toolbar),		UG_TYPE_INT,	NULL,	NULL},
+	{"Statusbar",	G_STRUCT_OFFSET (struct UgWindowSetting, statusbar),	UG_TYPE_INT,	NULL,	NULL},
+	{"Category",	G_STRUCT_OFFSET (struct UgWindowSetting, category),		UG_TYPE_INT,	NULL,	NULL},
+	{"Summary",		G_STRUCT_OFFSET (struct UgWindowSetting, summary),		UG_TYPE_INT,	NULL,	NULL},
+	{"x",			G_STRUCT_OFFSET (struct UgWindowSetting, x)	,			UG_TYPE_INT,	NULL,	NULL},
+	{"y",			G_STRUCT_OFFSET (struct UgWindowSetting, y),			UG_TYPE_INT,	NULL,	NULL},
+	{"width",		G_STRUCT_OFFSET (struct UgWindowSetting, width),		UG_TYPE_INT,	NULL,	NULL},
+	{"height",		G_STRUCT_OFFSET (struct UgWindowSetting, height),		UG_TYPE_INT,	NULL,	NULL},
+	{"maximized",	G_STRUCT_OFFSET (struct UgWindowSetting, maximized),	UG_TYPE_INT,	NULL,	NULL},
 	{NULL},			// null-terminated
 };
 
@@ -128,14 +128,18 @@ static const UgDataInterface	ug_window_setting_iface =
 // UgUserInterfaceSetting
 static const UgDataEntry	ug_user_interface_setting_entry[] =
 {
-	{"CloseConfirmation",	G_STRUCT_OFFSET (struct UgUserInterfaceSetting, close_confirmation),	UG_DATA_INT,	NULL,	NULL},
-	{"CloseAction",			G_STRUCT_OFFSET (struct UgUserInterfaceSetting, close_action),			UG_DATA_INT,	NULL,	NULL},
-	{"DeleteConfirmation",	G_STRUCT_OFFSET (struct UgUserInterfaceSetting,	delete_confirmation),	UG_DATA_INT,	NULL,	NULL},
-	{"ShowTrayIcon",		G_STRUCT_OFFSET (struct UgUserInterfaceSetting,	show_trayicon),			UG_DATA_INT,	NULL,	NULL},
-	{"StartInTray",			G_STRUCT_OFFSET (struct UgUserInterfaceSetting,	start_in_tray),			UG_DATA_INT,	NULL,	NULL},
-	{"StartInOfflineMode",	G_STRUCT_OFFSET (struct UgUserInterfaceSetting,	start_in_offline_mode),	UG_DATA_INT,	NULL,	NULL},
-	{"StartNotification",	G_STRUCT_OFFSET (struct UgUserInterfaceSetting,	start_notification),	UG_DATA_INT,	NULL,	NULL},
-	{"SoundNotification",	G_STRUCT_OFFSET (struct UgUserInterfaceSetting,	sound_notification),	UG_DATA_INT,	NULL,	NULL},
+	{"CloseConfirmation",	G_STRUCT_OFFSET (struct UgUserInterfaceSetting, close_confirmation),	UG_TYPE_INT,	NULL,	NULL},
+	{"CloseAction",			G_STRUCT_OFFSET (struct UgUserInterfaceSetting, close_action),			UG_TYPE_INT,	NULL,	NULL},
+	{"DeleteConfirmation",	G_STRUCT_OFFSET (struct UgUserInterfaceSetting,	delete_confirmation),	UG_TYPE_INT,	NULL,	NULL},
+	{"ShowTrayIcon",		G_STRUCT_OFFSET (struct UgUserInterfaceSetting,	show_trayicon),			UG_TYPE_INT,	NULL,	NULL},
+	{"StartInTray",			G_STRUCT_OFFSET (struct UgUserInterfaceSetting,	start_in_tray),			UG_TYPE_INT,	NULL,	NULL},
+	{"StartInOfflineMode",	G_STRUCT_OFFSET (struct UgUserInterfaceSetting,	start_in_offline_mode),	UG_TYPE_INT,	NULL,	NULL},
+	{"StartNotification",	G_STRUCT_OFFSET (struct UgUserInterfaceSetting,	start_notification),	UG_TYPE_INT,	NULL,	NULL},
+	{"SoundNotification",	G_STRUCT_OFFSET (struct UgUserInterfaceSetting,	sound_notification),	UG_TYPE_INT,	NULL,	NULL},
+	{"ApplyRecently",		G_STRUCT_OFFSET (struct UgUserInterfaceSetting,	apply_recently),		UG_TYPE_INT,	NULL,	NULL},
+#ifdef HAVE_APP_INDICATOR
+	{"AppIndicator",		G_STRUCT_OFFSET (struct UgUserInterfaceSetting,	app_indicator),			UG_TYPE_INT,	NULL,	NULL},
+#endif
 	{NULL},			// null-terminated
 };
 
@@ -151,10 +155,10 @@ static const UgDataInterface	ug_user_interface_setting_iface =
 // UgClipboardSetting
 static const UgDataEntry	ug_clipboard_setting_entry[] =
 {
-	{"pattern",		G_STRUCT_OFFSET (struct UgClipboardSetting, pattern),	UG_DATA_STRING,		NULL,	NULL},
-	{"monitor",		G_STRUCT_OFFSET (struct UgClipboardSetting, monitor),	UG_DATA_INT,		NULL,	NULL},
-	{"quiet",		G_STRUCT_OFFSET (struct UgClipboardSetting, quiet),		UG_DATA_INT,		NULL,	NULL},
-	{"NthCategory",	G_STRUCT_OFFSET (struct UgClipboardSetting, nth_category),	UG_DATA_INT,	NULL,	NULL},
+	{"pattern",		G_STRUCT_OFFSET (struct UgClipboardSetting, pattern),	UG_TYPE_STRING,		NULL,	NULL},
+	{"monitor",		G_STRUCT_OFFSET (struct UgClipboardSetting, monitor),	UG_TYPE_INT,		NULL,	NULL},
+	{"quiet",		G_STRUCT_OFFSET (struct UgClipboardSetting, quiet),		UG_TYPE_INT,		NULL,	NULL},
+	{"NthCategory",	G_STRUCT_OFFSET (struct UgClipboardSetting, nth_category),	UG_TYPE_INT,	NULL,	NULL},
 	{NULL},			// null-terminated
 };
 
@@ -166,13 +170,31 @@ static const UgDataInterface	ug_clipboard_setting_iface =
 	NULL, NULL, NULL,
 };
 
+// ----------------------------------------------------------------------------
+// UgSpeedLimitSetting
+static const UgDataEntry	ug_speed_limit_setting_entry[] =
+{
+	{"NormalUpload",	G_STRUCT_OFFSET (struct UgSpeedLimitSetting, normal.upload),	UG_TYPE_UINT,	NULL,	NULL},
+	{"NormalDownload",	G_STRUCT_OFFSET (struct UgSpeedLimitSetting, normal.download),	UG_TYPE_UINT,	NULL,	NULL},
+	{"SchedulerUpload",		G_STRUCT_OFFSET (struct UgSpeedLimitSetting, scheduler.upload),		UG_TYPE_UINT,	NULL,	NULL},
+	{"SchedulerDownload",	G_STRUCT_OFFSET (struct UgSpeedLimitSetting, scheduler.download),	UG_TYPE_UINT,	NULL,	NULL},
+	{NULL},			// null-terminated
+};
+
+static const UgDataInterface	ug_speed_limit_setting_iface =
+{
+	sizeof (struct UgSpeedLimitSetting),
+	"SpeedLimitSetting",
+	ug_speed_limit_setting_entry,
+	NULL, NULL, NULL,
+};
+
 // ----------------------------------------------------------------------------
 // SchedulerSetting
 static const UgDataEntry	ug_scheduler_setting_entry[] =
 {
-	{"enable",		G_STRUCT_OFFSET (struct UgSchedulerSetting, enable),	UG_DATA_INT,		NULL,	NULL},
-	{"state",		G_STRUCT_OFFSET (struct UgSchedulerSetting, state),		UG_DATA_CUSTOM,		ug_schedule_state_in_markup,	ug_schedule_state_to_markup},
-//	{"SpeedLimit",	G_STRUCT_OFFSET (struct UgSchedulerSetting, speed_limit),	UG_DATA_INT64,	NULL,	NULL},
+	{"enable",		G_STRUCT_OFFSET (struct UgSchedulerSetting, enable),	UG_TYPE_INT,		NULL,	NULL},
+	{"state",		G_STRUCT_OFFSET (struct UgSchedulerSetting, state),		UG_TYPE_CUSTOM,		ug_schedule_state_in_markup,	ug_schedule_state_to_markup},
 	{NULL},			// null-terminated
 };
 
@@ -188,12 +210,12 @@ static const UgDataInterface	ug_scheduler_setting_iface =
 // PluginSetting
 static const UgDataEntry	ug_plugin_setting_entry[] =
 {
-	{"aria2-enable",	G_STRUCT_OFFSET (struct UgPluginSetting, aria2.enable),		UG_DATA_INT,	NULL,	NULL},
-	{"aria2-launch",	G_STRUCT_OFFSET (struct UgPluginSetting, aria2.launch),		UG_DATA_INT,	NULL,	NULL},
-	{"aria2-shutdown",	G_STRUCT_OFFSET (struct UgPluginSetting, aria2.shutdown),	UG_DATA_INT,	NULL,	NULL},
-	{"aria2-path",		G_STRUCT_OFFSET (struct UgPluginSetting, aria2.path),		UG_DATA_STRING,	NULL,	NULL},
-	{"aria2-args",		G_STRUCT_OFFSET (struct UgPluginSetting, aria2.args),		UG_DATA_STRING,	NULL,	NULL},
-	{"aria2-uri",		G_STRUCT_OFFSET (struct UgPluginSetting, aria2.uri),		UG_DATA_STRING,	NULL,	NULL},
+	{"aria2-enable",	G_STRUCT_OFFSET (struct UgPluginSetting, aria2.enable),		UG_TYPE_INT,	NULL,	NULL},
+	{"aria2-launch",	G_STRUCT_OFFSET (struct UgPluginSetting, aria2.launch),		UG_TYPE_INT,	NULL,	NULL},
+	{"aria2-shutdown",	G_STRUCT_OFFSET (struct UgPluginSetting, aria2.shutdown),	UG_TYPE_INT,	NULL,	NULL},
+	{"aria2-path",		G_STRUCT_OFFSET (struct UgPluginSetting, aria2.path),		UG_TYPE_STRING,	NULL,	NULL},
+	{"aria2-args",		G_STRUCT_OFFSET (struct UgPluginSetting, aria2.args),		UG_TYPE_STRING,	NULL,	NULL},
+	{"aria2-uri",		G_STRUCT_OFFSET (struct UgPluginSetting, aria2.uri),		UG_TYPE_STRING,	NULL,	NULL},
 	{NULL},			// null-terminated
 };
 
@@ -205,24 +227,43 @@ static const UgDataInterface	ug_plugin_setting_iface =
 	NULL, NULL, NULL,
 };
 
+// ----------------------------------------------------------------------------
+// CommandlineSetting
+static const UgDataEntry	ug_commandline_setting_entry[] =
+{
+	{"quiet",			G_STRUCT_OFFSET (struct UgCommandlineSetting, quiet),			UG_TYPE_INT,	NULL,	NULL},
+	{"CategoryIndex",	G_STRUCT_OFFSET (struct UgCommandlineSetting, category_index),	UG_TYPE_INT,	NULL,	NULL},
+	{NULL},			// null-terminated
+};
+
+static const UgDataInterface	ug_commandline_setting_iface =
+{
+	sizeof (struct UgCommandlineSetting),
+	"CommandlineSetting",
+	ug_commandline_setting_entry,
+	NULL, NULL, NULL,
+};
+
 // ----------------------------------------------------------------------------
 // UgSetting
 static const UgDataEntry	uget_setting_data_entry[] =
 {
-	{"LaunchApp",		G_STRUCT_OFFSET (UgSetting, launch.active),		UG_DATA_INT,		NULL,	NULL},
-	{"LaunchAppTypes",	G_STRUCT_OFFSET (UgSetting, launch.types),		UG_DATA_STRING,		NULL,	NULL},
-	{"AutoSave",		G_STRUCT_OFFSET (UgSetting, auto_save.active),	UG_DATA_INT,		NULL,	NULL},
-	{"AutoSaveInterval",G_STRUCT_OFFSET (UgSetting, auto_save.interval),UG_DATA_INT,		NULL,	NULL},
-	{"DownloadColumn",	G_STRUCT_OFFSET (UgSetting, download_column),	UG_DATA_STATIC,		NULL,	NULL},
-	{"Summary",			G_STRUCT_OFFSET (UgSetting, summary),			UG_DATA_STATIC,		NULL,	NULL},
-	{"Window",			G_STRUCT_OFFSET (UgSetting, window),			UG_DATA_STATIC,		NULL,	NULL},
-	{"UserInterface",	G_STRUCT_OFFSET (UgSetting, ui),				UG_DATA_STATIC,		NULL,	NULL},
-	{"Clipboard",		G_STRUCT_OFFSET (UgSetting, clipboard),			UG_DATA_STATIC,		NULL,	NULL},
-	{"Scheduler",		G_STRUCT_OFFSET (UgSetting, scheduler),			UG_DATA_STATIC,		NULL,	NULL},
-	{"Plug-in",			G_STRUCT_OFFSET (UgSetting, plugin),			UG_DATA_STATIC,		NULL,	NULL},
-//	{"OfflineMode",		G_STRUCT_OFFSET (UgSetting, offline_mode),		UG_DATA_INT,		NULL,	NULL},
-//	{"Shutdown",		G_STRUCT_OFFSET (UgSetting, shutdown),			UG_DATA_INT,		NULL,	NULL},
-	{"FolderList",		G_STRUCT_OFFSET (UgSetting, folder_list),		UG_DATA_CUSTOM,		ug_string_list_in_markup,	ug_string_list_to_markup},
+	{"LaunchApp",		G_STRUCT_OFFSET (UgSetting, launch.active),		UG_TYPE_INT,		NULL,	NULL},
+	{"LaunchAppTypes",	G_STRUCT_OFFSET (UgSetting, launch.types),		UG_TYPE_STRING,		NULL,	NULL},
+	{"AutoSave",		G_STRUCT_OFFSET (UgSetting, auto_save.active),	UG_TYPE_INT,		NULL,	NULL},
+	{"AutoSaveInterval",G_STRUCT_OFFSET (UgSetting, auto_save.interval),UG_TYPE_INT,		NULL,	NULL},
+	{"DownloadColumn",	G_STRUCT_OFFSET (UgSetting, download_column),	UG_TYPE_STATIC,		NULL,	NULL},
+	{"Summary",			G_STRUCT_OFFSET (UgSetting, summary),			UG_TYPE_STATIC,		NULL,	NULL},
+	{"Window",			G_STRUCT_OFFSET (UgSetting, window),			UG_TYPE_STATIC,		NULL,	NULL},
+	{"UserInterface",	G_STRUCT_OFFSET (UgSetting, ui),				UG_TYPE_STATIC,		NULL,	NULL},
+	{"Clipboard",		G_STRUCT_OFFSET (UgSetting, clipboard),			UG_TYPE_STATIC,		NULL,	NULL},
+	{"SpeedLimit",		G_STRUCT_OFFSET (UgSetting, speed_limit),		UG_TYPE_STATIC,		NULL,	NULL},
+	{"Scheduler",		G_STRUCT_OFFSET (UgSetting, scheduler),			UG_TYPE_STATIC,		NULL,	NULL},
+	{"Commandline",		G_STRUCT_OFFSET (UgSetting, commandline),		UG_TYPE_STATIC,		NULL,	NULL},
+	{"Plug-in",			G_STRUCT_OFFSET (UgSetting, plugin),			UG_TYPE_STATIC,		NULL,	NULL},
+//	{"OfflineMode",		G_STRUCT_OFFSET (UgSetting, offline_mode),		UG_TYPE_INT,		NULL,	NULL},
+//	{"Shutdown",		G_STRUCT_OFFSET (UgSetting, shutdown),			UG_TYPE_INT,		NULL,	NULL},
+	{"FolderList",		G_STRUCT_OFFSET (UgSetting, folder_list),		UG_TYPE_CUSTOM,		ug_string_list_in_markup,	ug_string_list_to_markup},
 	{NULL},				// null-terminated
 };
 
@@ -431,6 +472,10 @@ void	ug_setting_init (UgSetting* setting)
 	setting->ui.start_in_offline_mode = FALSE;
 	setting->ui.start_notification = TRUE;
 	setting->ui.sound_notification = TRUE;
+	setting->ui.apply_recently = TRUE;
+#ifdef HAVE_APP_INDICATOR
+	setting->ui.app_indicator = TRUE;
+#endif
 
 	// "ClipboardSetting"
 	setting->clipboard.iface = &ug_clipboard_setting_iface;
@@ -440,6 +485,13 @@ void	ug_setting_init (UgSetting* setting)
 	setting->clipboard.quiet = FALSE;
 	setting->clipboard.nth_category = 0;
 
+	// "SpeedLimitSetting"
+	setting->speed_limit.iface = &ug_speed_limit_setting_iface;
+	setting->speed_limit.normal.upload = 0;
+	setting->speed_limit.normal.download = 0;
+	setting->speed_limit.scheduler.upload = 0;
+	setting->speed_limit.scheduler.download = 0;
+
 	// "SchedulerSetting"
 	setting->scheduler.iface = &ug_scheduler_setting_iface;
 	setting->scheduler.enable = FALSE;
@@ -447,7 +499,11 @@ void	ug_setting_init (UgSetting* setting)
 		for (dayhours = 0;  dayhours < 24;  dayhours++)
 			setting->scheduler.state[weekdays][dayhours] = UG_SCHEDULE_NORMAL;
 	}
-	setting->scheduler.speed_limit = 5;
+
+	// "CommandlineSetting"
+	setting->commandline.iface = &ug_commandline_setting_iface;
+	setting->commandline.quiet = FALSE;
+	setting->commandline.category_index = -1;
 
 	// "PluginSetting"
 	setting->plugin.iface = &ug_plugin_setting_iface;
@@ -455,7 +511,7 @@ void	ug_setting_init (UgSetting* setting)
 	setting->plugin.aria2.launch = TRUE;
 	setting->plugin.aria2.shutdown = TRUE;
 	setting->plugin.aria2.path = g_strdup ("aria2c");
-	setting->plugin.aria2.args = g_strdup ("--enable-rpc");
+	setting->plugin.aria2.args = g_strdup ("--enable-rpc=true -D --check-certificate=false");
 	setting->plugin.aria2.uri  = g_strdup ("http://localhost:6800/rpc");
 
 	// "FolderList"
