@@ -1,6 +1,6 @@
 /*
  *
- *   Copyright (C) 2005-2012 by C.H. Huang
+ *   Copyright (C) 2005-2013 by C.H. Huang
  *   plushuang.tw@gmail.com
  *
  *  This library is free software; you can redistribute it and/or
@@ -41,7 +41,7 @@
 #include <config.h>
 #endif
 #ifndef PACKAGE_VERSION
-#define PACKAGE_VERSION					"1.8.2"
+#define PACKAGE_VERSION					"1.10.3"
 #endif
 
 #include <UgData.h>
@@ -51,7 +51,7 @@ extern "C" {
 #endif
 
 #define UG_APP_GTK_NAME					"uGet"
-#define UG_APP_GTK_VERSION				PACKAGE_VERSION " (stable)"
+#define UG_APP_GTK_VERSION				PACKAGE_VERSION " (stable branch)"
 // default setting
 #define UG_APP_GTK_CLIPBOARD_PATTERN	"ZIP|BIN|GZ|7Z|Z|TAR|TGZ|BZ2|A[0-9]?|LZH|MP3|RPM|DEB|EXE|RAR|R[0-9][0-9]"
 #define UG_APP_GTK_LAUNCH_APP_TYPES		"torrent"
@@ -150,6 +150,10 @@ struct UgSetting
 		gboolean	start_in_offline_mode;
 		gboolean	start_notification;
 		gboolean	sound_notification;
+		gboolean	apply_recently;
+#ifdef HAVE_APP_INDICATOR
+		gboolean	app_indicator;
+#endif
 	} ui;
 
 	// "ClipboardSetting"
@@ -163,6 +167,22 @@ struct UgSetting
 		gint		nth_category;
 	} clipboard;
 
+	// "SpeedLimitSetting" - global speed limits
+	struct UgSpeedLimitSetting
+	{
+		const UgDataInterface*	iface;		// for UgMarkup parse/write
+
+		struct {
+			guint	upload;					// KiB / second
+			guint	download;				// KiB / second
+		} normal;
+
+		struct {
+			guint	upload;					// KiB / second
+			guint	download;				// KiB / second
+		} scheduler;
+	} speed_limit;
+
 	// "SchedulerSetting"
 	struct UgSchedulerSetting
 	{
@@ -170,9 +190,17 @@ struct UgSetting
 
 		gboolean	enable;
 		guint		state[7][24];	// 1 week, 7 days, 24 hours
-		gint64		speed_limit;
 	} scheduler;
 
+	// "CommandlineSetting"
+	struct UgCommandlineSetting
+	{
+		const UgDataInterface*	iface;		// for UgMarkup parse/write
+
+		gboolean	quiet;			// --quiet
+		gint		category_index;	// --category-index
+	} commandline;
+
 	// "PluginSetting"
 	struct UgPluginSetting
 	{
