@@ -1,6 +1,6 @@
 /*
  *
- *   Copyright (C) 2005-2012 by C.H. Huang
+ *   Copyright (C) 2005-2013 by C.H. Huang
  *   plushuang.tw@gmail.com
  *
  *  This library is free software; you can redistribute it and/or
@@ -65,7 +65,7 @@ void	ug_clipboard_setting_form_init (struct UgClipboardSettingForm* csform)
 	GtkBox*				vbox;
 	GtkBox*				hbox;
 
-	csform->self = gtk_vbox_new (FALSE, 0);
+	csform->self = gtk_box_new (GTK_ORIENTATION_VERTICAL, 0);
 	vbox = (GtkBox*) csform->self;
 	// Monitor button
 	widget = gtk_check_button_new_with_mnemonic (
@@ -81,12 +81,12 @@ void	ug_clipboard_setting_form_init (struct UgClipboardSettingForm* csform)
 	csform->pattern = (GtkEntry*) entry;
 
 	// tips
-	hbox = (GtkBox*) gtk_hbox_new (FALSE, 2);
+	hbox = (GtkBox*) gtk_box_new (GTK_ORIENTATION_HORIZONTAL, 2);
 	gtk_box_pack_start (vbox, (GtkWidget*) hbox, FALSE, FALSE, 1);
 	gtk_box_pack_end (hbox,
 			gtk_label_new (_("Separate the types with character '|'.")),
 			FALSE, FALSE, 2);
-	hbox = (GtkBox*) gtk_hbox_new (FALSE, 2);
+	hbox = (GtkBox*) gtk_box_new (GTK_ORIENTATION_HORIZONTAL, 2);
 	gtk_box_pack_start (vbox, (GtkWidget*) hbox, FALSE, FALSE, 1);
 	gtk_box_pack_end (hbox,
 			gtk_label_new (_("You can use regular expressions here.")),
@@ -95,7 +95,7 @@ void	ug_clipboard_setting_form_init (struct UgClipboardSettingForm* csform)
 	gtk_box_pack_start (vbox, gtk_label_new (""),
 			FALSE, FALSE, 2);
 	// quiet mode
-	hbox = (GtkBox*) gtk_hbox_new (FALSE, 2);
+	hbox = (GtkBox*) gtk_box_new (GTK_ORIENTATION_HORIZONTAL, 2);
 	gtk_box_pack_start (vbox, (GtkWidget*) hbox, FALSE, FALSE, 2);
 	widget = gtk_check_button_new_with_mnemonic (_("_Quiet mode"));
 	gtk_box_pack_start (hbox, widget, FALSE, FALSE, 0);
@@ -141,7 +141,7 @@ void	ug_user_interface_form_init (struct UgUserInterfaceForm* uiform)
 	GtkWidget*	widget;
 	GtkBox*		vbox;
 
-	uiform->self = gtk_vbox_new (FALSE, 0);
+	uiform->self = gtk_box_new (GTK_ORIENTATION_VERTICAL, 0);
 	vbox = (GtkBox*) uiform->self;
 	// check button
 	widget = gtk_check_button_new_with_label (_("Show confirmation dialog on close"));
@@ -165,6 +165,9 @@ void	ug_user_interface_form_init (struct UgUserInterfaceForm* uiform)
 	widget = gtk_check_button_new_with_label (_("Sound when download is finished"));
 	uiform->sound_notification = (GtkToggleButton*) widget;
 	gtk_box_pack_start (vbox, widget, FALSE, FALSE, 1);
+	widget = gtk_check_button_new_with_label (_("Apply recently download settings"));
+	uiform->apply_recently = (GtkToggleButton*) widget;
+	gtk_box_pack_start (vbox, widget, FALSE, FALSE, 1);
 }
 
 void	ug_user_interface_form_set (struct UgUserInterfaceForm* uiform, UgSetting* setting)
@@ -183,6 +186,8 @@ void	ug_user_interface_form_set (struct UgUserInterfaceForm* uiform, UgSetting*
 			setting->ui.start_notification);
 	gtk_toggle_button_set_active (uiform->sound_notification,
 			setting->ui.sound_notification);
+	gtk_toggle_button_set_active (uiform->apply_recently,
+			setting->ui.apply_recently);
 }
 
 void	ug_user_interface_form_get (struct UgUserInterfaceForm* uiform, UgSetting* setting)
@@ -194,6 +199,7 @@ void	ug_user_interface_form_get (struct UgUserInterfaceForm* uiform, UgSetting*
 	setting->ui.start_in_offline_mode = gtk_toggle_button_get_active (uiform->start_in_offline_mode);
 	setting->ui.start_notification = gtk_toggle_button_get_active (uiform->start_notification);
 	setting->ui.sound_notification = gtk_toggle_button_get_active (uiform->sound_notification);
+	setting->ui.apply_recently = gtk_toggle_button_get_active (uiform->apply_recently);
 }
 
 // ----------------------------------------------------------------------------
@@ -214,10 +220,10 @@ void	ug_launch_setting_form_init (struct UgLaunchSettingForm* lsform)
 	GtkBox*				vbox;
 	GtkBox*				hbox;
 
-	lsform->self = gtk_vbox_new (FALSE, 0);
+	lsform->self = gtk_box_new (GTK_ORIENTATION_VERTICAL, 0);
 	vbox = (GtkBox*) lsform->self;
 	// active button
-	hbox = (GtkBox*) gtk_hbox_new (FALSE, 0);
+	hbox = (GtkBox*) gtk_box_new (GTK_ORIENTATION_HORIZONTAL, 0);
 	gtk_box_pack_start (vbox, (GtkWidget*) hbox, FALSE, FALSE, 1);
 	widget = gtk_check_button_new_with_mnemonic (
 			_("_Launch default application for specified file types:"));
@@ -230,12 +236,12 @@ void	ug_launch_setting_form_init (struct UgLaunchSettingForm* lsform)
 	gtk_box_pack_start (vbox, entry, FALSE, FALSE, 2);
 	lsform->types = (GtkEntry*) entry;
 	// Launch app tips
-	hbox = (GtkBox*) gtk_hbox_new (FALSE, 2);
+	hbox = (GtkBox*) gtk_box_new (GTK_ORIENTATION_HORIZONTAL, 2);
 	gtk_box_pack_start (vbox, (GtkWidget*) hbox, FALSE, FALSE, 1);
 	gtk_box_pack_end (hbox,
 			gtk_label_new (_("Separate the types with character '|'.")),
 			FALSE, FALSE, 2);
-	hbox = (GtkBox*) gtk_hbox_new (FALSE, 2);
+	hbox = (GtkBox*) gtk_box_new (GTK_ORIENTATION_HORIZONTAL, 2);
 	gtk_box_pack_start (vbox, (GtkWidget*) hbox, FALSE, FALSE, 1);
 	gtk_box_pack_end (hbox,
 			gtk_label_new (_("You can use regular expressions here.")),
@@ -277,7 +283,7 @@ void	ug_auto_save_form_init (struct UgAutoSaveForm* asform)
 	GtkBox*				hbox;
 	GtkWidget*			widget;
 
-	asform->self = gtk_hbox_new (FALSE, 0);
+	asform->self = gtk_box_new (GTK_ORIENTATION_HORIZONTAL, 0);
 	hbox = (GtkBox*) asform->self;
 	widget = gtk_check_button_new_with_mnemonic (_("_Auto save"));
 	gtk_box_pack_start (hbox, widget, FALSE, FALSE, 0);
@@ -313,6 +319,51 @@ void	ug_auto_save_form_get (struct UgAutoSaveForm* asform, UgSetting* setting)
 	setting->auto_save.interval = gtk_spin_button_get_value_as_int (asform->interval_spin);
 }
 
+// ----------------------------------------------------------------------------
+// UgCommandlineSettingForm
+//
+void	ug_commandline_setting_form_init (struct UgCommandlineSettingForm* csform)
+{
+	GtkWidget*			widget;
+	GtkBox*				vbox;
+	GtkBox*				hbox;
+
+	csform->self = gtk_box_new (GTK_ORIENTATION_VERTICAL, 0);
+	vbox = (GtkBox*) csform->self;
+
+	// Commandline Settings
+	hbox = (GtkBox*) gtk_box_new (GTK_ORIENTATION_HORIZONTAL, 0);
+	gtk_box_pack_start (vbox, (GtkWidget*) hbox, FALSE, FALSE, 2);
+	widget = gtk_label_new (_("Commandline Settings"));
+	gtk_box_pack_start (hbox, widget, FALSE, FALSE, 2);
+	gtk_box_pack_start (hbox, gtk_separator_new (GTK_ORIENTATION_HORIZONTAL), TRUE, TRUE, 2);
+	// --quiet
+	widget = gtk_check_button_new_with_mnemonic (_("Use '--quiet' by default"));
+	gtk_box_pack_start (vbox, widget, FALSE, FALSE, 0);
+	csform->quiet = (GtkToggleButton*) widget;
+	// --category-index
+	hbox = (GtkBox*) gtk_box_new (GTK_ORIENTATION_HORIZONTAL, 2);
+	gtk_box_pack_start (vbox, (GtkWidget*) hbox, FALSE, FALSE, 2);
+	widget = gtk_label_new (_("'--category-index' default value"));
+	gtk_box_pack_start (hbox, widget, FALSE, FALSE, 2);
+	csform->index_label = widget;
+	widget = gtk_spin_button_new_with_range (-1.0, 100.0, 1.0);
+	gtk_box_pack_start (hbox, widget, FALSE, FALSE, 2);
+	csform->index_spin = (GtkSpinButton*) widget;
+}
+
+void	ug_commandline_setting_form_set (struct UgCommandlineSettingForm* csform, UgSetting* setting)
+{
+	gtk_toggle_button_set_active (csform->quiet, setting->commandline.quiet);
+	gtk_spin_button_set_value (csform->index_spin, setting->commandline.category_index);
+}
+
+void	ug_commandline_setting_form_get (struct UgCommandlineSettingForm* csform, UgSetting* setting)
+{
+	setting->commandline.quiet = gtk_toggle_button_get_active (csform->quiet);
+	setting->commandline.category_index = gtk_spin_button_get_value_as_int (csform->index_spin);
+}
+
 // ----------------------------------------------------------------------------
 // UgPluginSettingForm
 //
@@ -324,11 +375,14 @@ static void	on_plugin_aria2_toggled (GtkWidget* widget, struct UgPluginSettingFo
 	gtk_widget_set_sensitive ((GtkWidget*) psform->launch, sensitive);
 	gtk_widget_set_sensitive ((GtkWidget*) psform->shutdown, sensitive);
 	gtk_widget_set_sensitive ((GtkWidget*) psform->uri, sensitive);
+	gtk_widget_set_sensitive ((GtkWidget*) psform->upload, sensitive);
+	gtk_widget_set_sensitive ((GtkWidget*) psform->download, sensitive);
 
 	if (sensitive)
 		sensitive = gtk_toggle_button_get_active (psform->launch);
 	gtk_widget_set_sensitive ((GtkWidget*) psform->path, sensitive);
 	gtk_widget_set_sensitive ((GtkWidget*) psform->args, sensitive);
+	gtk_widget_set_sensitive ((GtkWidget*) psform->hint, sensitive);
 }
 
 static void	on_plugin_aria2_launch_toggled (GtkWidget* widget, struct UgPluginSettingForm* psform)
@@ -338,6 +392,7 @@ static void	on_plugin_aria2_launch_toggled (GtkWidget* widget, struct UgPluginSe
 	sensitive = gtk_toggle_button_get_active (psform->launch);
 	gtk_widget_set_sensitive ((GtkWidget*) psform->path, sensitive);
 	gtk_widget_set_sensitive ((GtkWidget*) psform->args, sensitive);
+	gtk_widget_set_sensitive ((GtkWidget*) psform->hint, sensitive);
 }
 
 void	ug_plugin_setting_form_init (struct UgPluginSettingForm* psform)
@@ -346,36 +401,19 @@ void	ug_plugin_setting_form_init (struct UgPluginSettingForm* psform)
 	GtkBox*				hbox;
 	GtkWidget*			widget;
 
-	psform->self = gtk_vbox_new (FALSE, 0);
+	psform->self = gtk_box_new (GTK_ORIENTATION_VERTICAL, 0);
 	vbox = (GtkBox*) psform->self;
 
-	hbox = (GtkBox*) gtk_hbox_new (FALSE, 0);
+	// Enable aria2 plug-in
+	hbox = (GtkBox*) gtk_box_new (GTK_ORIENTATION_HORIZONTAL, 0);
 	gtk_box_pack_start (vbox, (GtkWidget*) hbox, FALSE, TRUE, 2);
-
 	widget = gtk_check_button_new_with_mnemonic (_("_Enable aria2 plug-in"));
-	gtk_box_pack_start (hbox, widget, FALSE, FALSE, 2);
-	gtk_box_pack_start (hbox, gtk_hseparator_new (), TRUE, TRUE, 2);
+	gtk_box_pack_start (hbox, widget, FALSE, FALSE, 0);
+	gtk_box_pack_start (hbox, gtk_separator_new (GTK_ORIENTATION_HORIZONTAL), TRUE, TRUE, 4);
 	g_signal_connect (widget, "toggled",
 			G_CALLBACK (on_plugin_aria2_toggled), psform);
 	psform->enable = (GtkToggleButton*) widget;
 
-	// aria2 plug-in hints
-	widget = gtk_label_new (_("Please make sure that all tasks have been completed\n" "before you modify settings."));
-	gtk_box_pack_start (vbox, widget, FALSE, FALSE, 3);
-	widget = gtk_label_new ("");
-	gtk_box_pack_start (vbox, widget, FALSE, FALSE, 3);
-
-	// URI entry
-	hbox = (GtkBox*) gtk_hbox_new (FALSE, 0);
-	gtk_box_pack_start (vbox, (GtkWidget*) hbox, FALSE, TRUE, 2);
-	widget = gtk_label_new ("URI");
-	gtk_box_pack_start (hbox, widget, FALSE, FALSE, 2);
-	widget = gtk_entry_new ();
-	gtk_box_pack_start (hbox, widget, TRUE,  TRUE,  4);
-	psform->uri = (GtkEntry*) widget;
-
-	gtk_box_pack_start (vbox, gtk_hseparator_new (), FALSE, FALSE, 6);
-
 	widget = gtk_check_button_new_with_mnemonic (_("_Launch aria2 on startup"));
 	gtk_box_pack_start (vbox, widget, FALSE, FALSE, 2);
 	g_signal_connect (widget, "toggled",
@@ -386,16 +424,25 @@ void	ug_plugin_setting_form_init (struct UgPluginSettingForm* psform)
 	gtk_box_pack_start (vbox, widget, FALSE, FALSE, 2);
 	psform->shutdown = (GtkToggleButton*) widget;
 
-	// path
-	hbox = (GtkBox*) gtk_hbox_new (FALSE, 0);
+	// URI entry
+	hbox = (GtkBox*) gtk_box_new (GTK_ORIENTATION_HORIZONTAL, 0);
+	gtk_box_pack_start (vbox, (GtkWidget*) hbox, FALSE, TRUE, 2);
+	widget = gtk_label_new ("URI");
+	gtk_box_pack_start (hbox, widget, FALSE, FALSE, 2);
+	widget = gtk_entry_new ();
+	gtk_box_pack_start (hbox, widget, TRUE,  TRUE,  4);
+	psform->uri = (GtkEntry*) widget;
+
+	// Path
+	hbox = (GtkBox*) gtk_box_new (GTK_ORIENTATION_HORIZONTAL, 0);
 	gtk_box_pack_start (vbox, (GtkWidget*) hbox, FALSE, TRUE, 2);
 	widget = gtk_label_new (_("Path"));
 	gtk_box_pack_start (hbox, widget, FALSE, FALSE, 2);
 	widget = gtk_entry_new ();
 	gtk_box_pack_start (hbox, widget, TRUE,  TRUE,  4);
 	psform->path = (GtkEntry*) widget;
-	// argument
-	hbox = (GtkBox*) gtk_hbox_new (FALSE, 0);
+	// Arguments
+	hbox = (GtkBox*) gtk_box_new (GTK_ORIENTATION_HORIZONTAL, 0);
 	gtk_box_pack_start (vbox, (GtkWidget*) hbox, FALSE, TRUE, 2);
 	widget = gtk_label_new (_("Arguments"));
 	gtk_box_pack_start (hbox, widget, FALSE, FALSE, 2);
@@ -403,6 +450,40 @@ void	ug_plugin_setting_form_init (struct UgPluginSettingForm* psform)
 	gtk_box_pack_start (hbox, widget, TRUE,  TRUE,  4);
 	psform->args = (GtkEntry*) widget;
 
+	// aria2 plug-in hint
+	widget = gtk_label_new (_("You must restart uGet after modifying 'Arguments'."));
+	gtk_box_pack_start (vbox, widget, FALSE, FALSE, 2);
+	psform->hint = widget;
+
+	// Speed Limits
+	hbox = (GtkBox*) gtk_box_new (GTK_ORIENTATION_HORIZONTAL, 0);
+	gtk_box_pack_start (vbox, (GtkWidget*) hbox, FALSE, TRUE, 2);
+	widget = gtk_label_new (_("Speed Limits"));
+	gtk_box_pack_start (hbox, widget, FALSE, FALSE, 2);
+	gtk_box_pack_start (hbox, gtk_separator_new (GTK_ORIENTATION_HORIZONTAL), TRUE, TRUE, 2);
+	// Upload
+	hbox = (GtkBox*) gtk_box_new (GTK_ORIENTATION_HORIZONTAL, 2);
+	gtk_box_pack_start (vbox, (GtkWidget*) hbox, FALSE, FALSE, 2);
+	widget = gtk_label_new (_("Upload"));
+	gtk_box_pack_start (hbox, widget, FALSE, FALSE, 2);
+	widget = gtk_label_new (_("KiB/s"));
+	gtk_box_pack_end (hbox, widget, FALSE, FALSE, 2);
+	widget = gtk_spin_button_new_with_range (0.0, 4000000000.0, 5.0);
+	gtk_entry_set_width_chars (GTK_ENTRY (widget), 15);
+	gtk_box_pack_end (hbox, widget, FALSE, FALSE, 2);
+	psform->upload = (GtkSpinButton*) widget;
+	// Download
+	hbox = (GtkBox*) gtk_box_new (GTK_ORIENTATION_HORIZONTAL, 2);
+	gtk_box_pack_start (vbox, (GtkWidget*) hbox, FALSE, FALSE, 2);
+	widget = gtk_label_new (_("Download"));
+	gtk_box_pack_start (hbox, widget, FALSE, FALSE, 2);
+	widget = gtk_label_new (_("KiB/s"));
+	gtk_box_pack_end (hbox, widget, FALSE, FALSE, 2);
+	widget = gtk_spin_button_new_with_range (0.0, 4000000000.0, 5.0);
+	gtk_entry_set_width_chars (GTK_ENTRY (widget), 15);
+	gtk_box_pack_end (hbox, widget, FALSE, FALSE, 2);
+	psform->download = (GtkSpinButton*) widget;
+
 	gtk_widget_show (psform->self);
 }
 
@@ -416,6 +497,9 @@ void	ug_plugin_setting_form_set (struct UgPluginSettingForm* psform, UgSetting*
 	gtk_entry_set_text (psform->path, setting->plugin.aria2.path);
 	gtk_entry_set_text (psform->args, setting->plugin.aria2.args);
 
+	gtk_spin_button_set_value (psform->upload, setting->speed_limit.normal.upload);
+	gtk_spin_button_set_value (psform->download, setting->speed_limit.normal.download);
+
 	on_plugin_aria2_toggled ((GtkWidget*) psform->enable, psform);
 }
 
@@ -428,5 +512,8 @@ void	ug_plugin_setting_form_get (struct UgPluginSettingForm* psform, UgSetting*
 	setting->plugin.aria2.uri  = g_strdup (gtk_entry_get_text (psform->uri));
 	setting->plugin.aria2.path = g_strdup (gtk_entry_get_text (psform->path));
 	setting->plugin.aria2.args = g_strdup (gtk_entry_get_text (psform->args));
+
+	setting->speed_limit.normal.upload   = (guint) gtk_spin_button_get_value (psform->upload);
+	setting->speed_limit.normal.download = (guint) gtk_spin_button_get_value (psform->download);
 }
 
