@@ -1,6 +1,6 @@
 /*
  *
- *   Copyright (C) 2005-2012 by C.H. Huang
+ *   Copyright (C) 2005-2013 by C.H. Huang
  *   plushuang.tw@gmail.com
  *
  *  This library is free software; you can redistribute it and/or
@@ -55,12 +55,12 @@ static void	ug_int_list_to_markup (GList** list, UgMarkup* markup);
 
 static const UgDataEntry	ug_category_entry[] =
 {
-	{"name",			G_STRUCT_OFFSET (UgCategory, name),				UG_DATA_STRING,		NULL,	NULL},
-	{"ActiveLimit",		G_STRUCT_OFFSET (UgCategory, active_limit),		UG_DATA_UINT,		NULL,	NULL},
-	{"FinishedLimit",	G_STRUCT_OFFSET (UgCategory, finished_limit),	UG_DATA_UINT,		NULL,	NULL},
-	{"RecycledLimit",	G_STRUCT_OFFSET (UgCategory, recycled_limit),	UG_DATA_UINT,		NULL,	NULL},
-	{"DownloadDefault",	G_STRUCT_OFFSET (UgCategory, defaults),			UG_DATA_INSTANCE,	&ug_dataset_iface,	NULL},
-	{"DownloadIndices",	G_STRUCT_OFFSET (UgCategory, indices),			UG_DATA_CUSTOM,		ug_int_list_in_markup,	ug_int_list_to_markup},
+	{"name",			G_STRUCT_OFFSET (UgCategory, name),				UG_TYPE_STRING,		NULL,	NULL},
+	{"ActiveLimit",		G_STRUCT_OFFSET (UgCategory, active_limit),		UG_TYPE_UINT,		NULL,	NULL},
+	{"FinishedLimit",	G_STRUCT_OFFSET (UgCategory, finished_limit),	UG_TYPE_UINT,		NULL,	NULL},
+	{"RecycledLimit",	G_STRUCT_OFFSET (UgCategory, recycled_limit),	UG_TYPE_UINT,		NULL,	NULL},
+	{"DownloadDefault",	G_STRUCT_OFFSET (UgCategory, defaults),			UG_TYPE_INSTANCE,	&ug_dataset_iface,	NULL},
+	{"DownloadIndices",	G_STRUCT_OFFSET (UgCategory, indices),			UG_TYPE_CUSTOM,		ug_int_list_in_markup,	ug_int_list_to_markup},
 	{NULL},			// null-terminated
 };
 // extern
@@ -419,7 +419,6 @@ static GMarkupParser	ug_download_list_parser =
 
 GList*	ug_download_list_load (const gchar* download_file)
 {
-	UgDataCommon*	common;
 	UgRelation*		relation;
 	GList*			list;
 	GList*			link;
@@ -429,19 +428,6 @@ GList*	ug_download_list_load (const gchar* download_file)
 	// attachment
 	for (link = list;  link;  link = link->next) {
 		relation = UG_DATASET_RELATION ((UgDataset*) link->data);
-
-		// Compatibility code for old DownloadList.xml
-		if (relation == NULL)
-			relation = ug_dataset_alloc_front (link->data, UG_RELATION_I);
-		common = UG_DATASET_COMMON ((UgDataset*) link->data);
-		if (relation->attached.stamp == 0 && common) {
-			relation->attached.stamp = common->attached.stamp;
-			ug_str_set (&relation->attached.folder, common->attached.folder, -1);
-			g_free (common->attached.folder);
-			common->attached.folder = NULL;
-			common->attached.stamp = 0;
-		}
-
 		ug_attachment_ref (relation->attached.stamp);
 	}
 
@@ -612,13 +598,13 @@ void	ug_download_complete_data (UgDataset* dataset)
 	if (ug_uri_full_init (&urifull, common->url) == 0)
 		return;
 	// file
-	if (common->file == NULL) {
-		string = ug_uri_full_get_file (&urifull);
-		if (string)
-			common->file = (gchar*) string;
-		else
-			common->file = g_strdup ("index.htm");
-	}
+//	if (common->file == NULL) {
+//		string = ug_uri_full_get_file (&urifull);
+//		if (string)
+//			common->file = (gchar*) string;
+//		else
+//			common->file = g_strdup ("index.htm");
+//	}
 	// user
 	if (common->user == NULL) {
 		length = ug_uri_full_user (&urifull, &string);
@@ -647,11 +633,11 @@ static void	ug_relation_assign   (UgRelation* relation, UgRelation* src);
 
 static const UgDataEntry	ug_relation_entry[] =
 {
-	{"hints",			G_STRUCT_OFFSET (UgRelation, hints),			UG_DATA_UINT,	NULL,	NULL},
-	{"AttachedFolder",	G_STRUCT_OFFSET (UgRelation, attached.folder),	UG_DATA_STRING,	NULL,	NULL},
-	{"AttachedStamp",	G_STRUCT_OFFSET (UgRelation, attached.stamp),	UG_DATA_UINT,	NULL,	NULL},
-	{"MessageType",		G_STRUCT_OFFSET (UgRelation, message.type),		UG_DATA_UINT,	NULL,	NULL},
-	{"MessageString",	G_STRUCT_OFFSET (UgRelation, message.string),	UG_DATA_STRING,	NULL,	NULL},
+	{"hints",			G_STRUCT_OFFSET (UgRelation, hints),			UG_TYPE_UINT,	NULL,	NULL},
+	{"AttachedFolder",	G_STRUCT_OFFSET (UgRelation, attached.folder),	UG_TYPE_STRING,	NULL,	NULL},
+	{"AttachedStamp",	G_STRUCT_OFFSET (UgRelation, attached.stamp),	UG_TYPE_UINT,	NULL,	NULL},
+	{"MessageType",		G_STRUCT_OFFSET (UgRelation, message.type),		UG_TYPE_UINT,	NULL,	NULL},
+	{"MessageString",	G_STRUCT_OFFSET (UgRelation, message.string),	UG_TYPE_STRING,	NULL,	NULL},
 	{NULL},			// null-terminated
 };
 // extern
