@@ -1,6 +1,6 @@
 /*
  *
- *   Copyright (C) 2005-2012 by C.H. Huang
+ *   Copyright (C) 2005-2013 by C.H. Huang
  *   plushuang.tw@gmail.com
  *
  *  This library is free software; you can redistribute it and/or
@@ -154,7 +154,7 @@ static void	ug_plugin_aria2_global_finalize (void)
 
 static UgResult	ug_plugin_aria2_global_set (guint parameter, gpointer data)
 {
-	if (parameter != UG_DATA_STRING)
+	if (parameter != UG_TYPE_STRING)
 		return UG_RESULT_UNSUPPORT;
 
 	g_free (xmlrpc_uri);
@@ -220,7 +220,7 @@ static UgResult	ug_plugin_aria2_set_state (UgPluginAria2* plugin, UgState  state
 		if (state == UG_STATE_ACTIVE && old_state < UG_STATE_ACTIVE) {
 			// call ug_plugin_unref () by ug_plugin_aria2_thread ()
 			ug_plugin_ref ((UgPlugin*) plugin);
-			g_thread_create ((GThreadFunc) ug_plugin_aria2_thread, plugin, FALSE, NULL);
+			g_thread_new ("uget-aria2", (GThreadFunc) ug_plugin_aria2_thread, plugin);
 		}
 
 		ug_plugin_post ((UgPlugin*) plugin, ug_message_new_state (state));
@@ -245,7 +245,7 @@ UgResult	ug_plugin_aria2_set (UgPluginAria2* plugin, guint parameter, gpointer d
 	UgXmlrpcValue*	member;
 	gint64			speed_limit;
 
-	if (parameter != UG_DATA_INT64)
+	if (parameter != UG_TYPE_INT64)
 		return UG_RESULT_UNSUPPORT;
 
 	speed_limit = *(gint64*)data;
@@ -274,7 +274,7 @@ static UgResult	ug_plugin_aria2_get (UgPluginAria2* plugin, guint parameter, gpo
 {
 	UgProgress*	progress;
 
-	if (parameter != UG_DATA_INSTANCE)
+	if (parameter != UG_TYPE_INSTANCE)
 		return UG_RESULT_UNSUPPORT;
 	if (data == NULL || ((UgData*)data)->iface != UG_PROGRESS_I)
 		return UG_RESULT_UNSUPPORT;
@@ -308,16 +308,18 @@ static UgResult	ug_plugin_aria2_get (UgPluginAria2* plugin, guint parameter, gpo
 //
 static gpointer	ug_plugin_aria2_thread (UgPluginAria2* plugin)
 {
-	UgMessage*	message;
-	char*		string;
-	const char*	temp;
-	char		ext;
-	time_t		startingTime;
-	gboolean	redirection;
+	UgDataCommon*	common;
+	UgMessage*		message;
+	char*			string;
+	const char*		temp;
+	char			ext;
+	time_t			startingTime;
+	gboolean		redirection;
 
 	g_string_chunk_clear (plugin->chunk);
 	startingTime = time (NULL);
 	redirection  = TRUE;
+	common = plugin->common;
 
 	if (ug_plugin_aria2_get_version (plugin) == FALSE)
 		goto exit;
@@ -379,6 +381,16 @@ static gpointer	ug_plugin_aria2_thread (UgPluginAria2* plugin)
 
 		switch (plugin->aria2Status) {
 		case ARIA2_COMPLETE:
+			if (plugin->renamed == FALSE && plugin->common->file && plugin->local_file == NULL) {
+				string = g_strdup_printf ("%.*s%s",
+						plugin->path_folder_len, plugin->path, plugin->common->file);
+				if (ug_rename (plugin->path, string) == 0) {
+					ug_plugin_post ((UgPlugin*) plugin,
+							ug_message_new_warning (UG_MESSAGE_WARNING_FILE_RENAME_FAILED, NULL));
+				}
+				g_free (string);
+				plugin->renamed = TRUE;
+			}
 			// if current download is metalink or torrent file
 			if (plugin->followed) {
 				if (plugin->gid == plugin->followed->data)
@@ -396,8 +408,34 @@ static gpointer	ug_plugin_aria2_thread (UgPluginAria2* plugin)
 			goto break_while;
 
 		case ARIA2_ERROR:
-			ug_plugin_aria2_remove (plugin);
-			ug_plugin_aria2_post_error (plugin, plugin->errorCode);
+			// tell aria2 to remove this gid
+//			ug_plugin_aria2_remove (plugin);
+			// download speed was too slow
+			if (plugin->errorCode == 5 && plugin->local_file == NULL) {
+				// retry
+				if (common->retry_count < common->retry_limit) {
+					common->retry_count++;
+					ug_plugin_post ((UgPlugin*) plugin,
+							ug_message_new_info (UG_MESSAGE_INFO_RETRY, NULL));
+					ug_plugin_delay ((UgPlugin*) plugin, common->retry_delay * 1000);
+					// clear data
+					g_string_chunk_clear (plugin->chunk);
+					g_list_free (plugin->followed);
+					plugin->followed = NULL;
+					// restart download
+					if (ug_plugin_aria2_add_uri (plugin) == FALSE)
+						goto exit;
+					break;
+				}
+				else {
+					ug_plugin_post ((UgPlugin*) plugin,
+							ug_message_new_error (UG_MESSAGE_ERROR_TOO_MANY_RETRIES, NULL));
+				}
+			}
+			else {
+				// post other error message
+				ug_plugin_aria2_post_error (plugin, plugin->errorCode);
+			}
 			goto break_while;
 		}
 
@@ -428,14 +466,30 @@ static gboolean	ug_plugin_aria2_add_uri	(UgPluginAria2* plugin)
 	UgXmlrpcValue*		value;
 	UgDataCommon*		common;
 	UgXmlrpcResponse	response;
+	char*				curr;
+	char*				prev;
 
 	common = plugin->common;
 	// URIs array
 	ug_plugin_aria2_set_scheme (plugin);
-	uris = ug_xmlrpc_value_new_array (1);
+	uris = ug_xmlrpc_value_new_array (6);
 	value = ug_xmlrpc_value_alloc (uris);
 	value->type = UG_XMLRPC_STRING;
 	value->c.string = common->url;
+	// URIs array (mirrors)
+	for (curr = common->mirrors;  curr && curr[0];  curr = curr + 1) {
+		// skip space ' '
+		while (curr[0] == ' ')
+			curr++;
+		prev = curr;
+		curr = curr + strcspn (curr, " ");
+
+		value = ug_xmlrpc_value_alloc (uris);
+		value->type = UG_XMLRPC_STRING;
+		value->c.string = g_string_chunk_insert_len (plugin->chunk,
+				prev, curr - prev);
+	}
+
 	// options struct
 	options = ug_xmlrpc_value_new_struct (16);
 	ug_plugin_aria2_set_common (plugin, options);
@@ -736,7 +790,8 @@ static gboolean	ug_plugin_aria2_tell_status (UgPluginAria2* plugin)
 			plugin->path = g_string_chunk_insert (plugin->chunk, keys->c.string);
 			string = strrchr (plugin->path, '/');
 			string = string ? string+1 : plugin->path;
-			if (string[0]) {
+			plugin->path_folder_len = string - plugin->path;
+			if (string[0] && plugin->common->file == NULL) {
 				ug_plugin_post ((UgPlugin*) plugin,
 						ug_message_new_data (UG_MESSAGE_DATA_FILE_CHANGED, string));
 			}
@@ -795,10 +850,22 @@ static void	ug_plugin_aria2_post_error (UgPluginAria2* plugin, int code)
 	gchar*			string;
 
 	switch (code) {
+	case 1:
+		message = ug_message_new_error (UG_MESSAGE_ERROR_CUSTOM, "aria2: an unknown error occurred.");
+		break;
+
+	case 2:
+		message = ug_message_new_error (UG_MESSAGE_ERROR_CUSTOM, "aria2: time out occurred.");
+		break;
+
 	case 3:
 		message = ug_message_new_error (UG_MESSAGE_ERROR_CUSTOM, "aria2: resource was not found.");
 		break;
 
+	case 4:
+		message = ug_message_new_error (UG_MESSAGE_ERROR_CUSTOM, "aria2 saw the specfied number of 'resource not found' error. See --max-file-not-found option");
+		break;
+
 	case 5:
 		message = ug_message_new_error (UG_MESSAGE_ERROR_CUSTOM, "aria2: speed was too slow.");
 		break;
@@ -819,6 +886,10 @@ static void	ug_plugin_aria2_post_error (UgPluginAria2* plugin, int code)
 		message = ug_message_new_error (UG_MESSAGE_ERROR_OUT_OF_RESOURCE, NULL);
 		break;
 
+	case 10:
+		message = ug_message_new_error (UG_MESSAGE_ERROR_CUSTOM, "aria2: piece length was different from one in .aria2 control file.");
+		break;
+
 	case 11:
 		message = ug_message_new_error (UG_MESSAGE_ERROR_CUSTOM, "aria2 was downloading same file.");
 		break;
@@ -827,14 +898,38 @@ static void	ug_plugin_aria2_post_error (UgPluginAria2* plugin, int code)
 		message = ug_message_new_error (UG_MESSAGE_ERROR_CUSTOM, "aria2 was downloading same info hash torrent.");
 		break;
 
+	case 13:
+		message = ug_message_new_error (UG_MESSAGE_ERROR_CUSTOM, "aria2: file already existed. See --allow-overwrite option.");
+		break;
+
 	case 14:
 		message = ug_message_new_error (UG_MESSAGE_ERROR_CUSTOM, _("Output file can't be renamed."));
 		break;
 
+	case 15:
+		message = ug_message_new_error (UG_MESSAGE_ERROR_CUSTOM, "aria2: could not open existing file.");
+		break;
+
+	case 16:
+		message = ug_message_new_error (UG_MESSAGE_ERROR_CUSTOM, "aria2: could not create new file or truncate existing file.");
+		break;
+
+	case 17:
+		message = ug_message_new_error (UG_MESSAGE_ERROR_CUSTOM, "aria2: file I/O error occurred.");
+		break;
+
 	case 18:
 		message = ug_message_new_error (UG_MESSAGE_ERROR_FOLDER_CREATE_FAILED, NULL);
 		break;
 
+	case 19:
+		message = ug_message_new_error (UG_MESSAGE_ERROR_CUSTOM, "aria2: name resolution failed.");
+		break;
+
+	case 20:
+		message = ug_message_new_error (UG_MESSAGE_ERROR_CUSTOM, "aria2: could not parse Metalink document.");
+		break;
+
 	case 21:
 		message = ug_message_new_error (UG_MESSAGE_ERROR_CUSTOM, "aria2: FTP command failed.");
 		break;
@@ -851,10 +946,35 @@ static void	ug_plugin_aria2_post_error (UgPluginAria2* plugin, int code)
 		message = ug_message_new_error (UG_MESSAGE_ERROR_CUSTOM, "aria2: HTTP authorization failed.");
 		break;
 
+	case 25:
+		message = ug_message_new_error (UG_MESSAGE_ERROR_CUSTOM, "aria2: could not parse bencoded file(usually .torrent file).");
+		break;
+
+	case 26:
+		message = ug_message_new_error (UG_MESSAGE_ERROR_CUSTOM, "aria2: torrent file was corrupted or missing information.");
+		break;
+
+	case 27:
+		message = ug_message_new_error (UG_MESSAGE_ERROR_CUSTOM, "aria2: Magnet URI was bad.");
+		break;
+
+	case 28:
+		message = ug_message_new_error (UG_MESSAGE_ERROR_CUSTOM, "aria2: bad/unrecognized option was given or unexpected option argument was given.");
+		break;
+
+	case 29:
+		message = ug_message_new_error (UG_MESSAGE_ERROR_CUSTOM, "aria2: remote server was unable to handle the request.");
+		break;
+
+	case 30:
+		message = ug_message_new_error (UG_MESSAGE_ERROR_CUSTOM, "aria2: could not parse JSON-RPC request.");
+		break;
+
 	default:
 		string = g_strdup_printf ("aria2 error code: %u", plugin->errorCode);
 		message = ug_message_new_error (UG_MESSAGE_ERROR_CUSTOM, string);
 		g_free (string);
+		break;
 	}
 
 	ug_plugin_post ((UgPlugin*) plugin, message);
@@ -933,12 +1053,19 @@ static void	ug_plugin_aria2_set_common	(UgPluginAria2* plugin, UgXmlrpcValue* op
 		value->type = UG_XMLRPC_STRING;
 		value->c.string = common->folder;
 	}
-	// max-tries
+	// continue
 	value = ug_xmlrpc_value_alloc (options);
-	value->name = "max-tries";
+	value->name = "continue";
 	value->type = UG_XMLRPC_STRING;
-	g_string_printf (string, "%u", common->retry_limit);
-	value->c.string = g_string_chunk_insert (plugin->chunk, string->str);
+	value->c.string = "true";
+	// remote-time
+	value = ug_xmlrpc_value_alloc (options);
+	value->name = "remote-time";
+	value->type = UG_XMLRPC_STRING;
+	if (common->retrieve_timestamp)
+		value->c.string = "true";
+	else
+		value->c.string = "false";
 	// retry-wait (since aria2 v1.11.0)
 	if (plugin->major_version >= 1  &&  plugin->minor_version >= 11) {
 		value = ug_xmlrpc_value_alloc (options);
@@ -947,6 +1074,12 @@ static void	ug_plugin_aria2_set_common	(UgPluginAria2* plugin, UgXmlrpcValue* op
 		g_string_printf (string, "%u", common->retry_delay);
 		value->c.string = g_string_chunk_insert (plugin->chunk, string->str);
 	}
+	// max-tries
+	value = ug_xmlrpc_value_alloc (options);
+	value->name = "max-tries";
+	value->type = UG_XMLRPC_STRING;
+	g_string_printf (string, "%u", common->retry_limit);
+	value->c.string = g_string_chunk_insert (plugin->chunk, string->str);
 	// max-download-limit
 	value = ug_xmlrpc_value_alloc (options);
 	value->name = "max-download-limit";
