@@ -1,6 +1,6 @@
 /*
  *
- *   Copyright (C) 2005-2012 by C.H. Huang
+ *   Copyright (C) 2005-2013 by C.H. Huang
  *   plushuang.tw@gmail.com
  *
  *  This library is free software; you can redistribute it and/or
@@ -72,6 +72,8 @@ struct UgPluginAria2
 	UgDataHttp*		http;
 	UgDataFtp*		ftp;
 
+	gboolean		renamed;
+
 	// temp
 	GString*		string;
 	GStringChunk*	chunk;
@@ -85,6 +87,7 @@ struct UgPluginAria2
 	gchar*			gid;
 	GList*			followed;	// followedBy
 	gchar*			path;
+	guint			path_folder_len;
 	guint			errorCode;
 	guint			aria2Status;
 	gint64			totalLength;
