@@ -1,6 +1,6 @@
 /*
  *
- *   Copyright (C) 2005-2012 by C.H. Huang
+ *   Copyright (C) 2005-2013 by C.H. Huang
  *   plushuang.tw@gmail.com
  *
  *  This library is free software; you can redistribute it and/or
@@ -188,7 +188,7 @@ static UgResult	ug_plugin_curl_set_state (UgPluginCurl* plugin, UgState  state)
 	if (state != old_state) {
 		if (state == UG_STATE_ACTIVE && old_state < UG_STATE_ACTIVE) {
 			ug_plugin_ref ((UgPlugin*) plugin);		// call ug_plugin_unref () by ug_plugin_curl_thread ()
-			g_thread_create ((GThreadFunc) ug_plugin_curl_thread, plugin, FALSE, NULL);
+			g_thread_new ("uget-curl", (GThreadFunc) ug_plugin_curl_thread, plugin);
 		}
 
 		ug_plugin_post ((UgPlugin*) plugin, ug_message_new_state (state));
@@ -211,7 +211,7 @@ static UgResult	ug_plugin_curl_set (UgPluginCurl* plugin, guint parameter, gpoin
 {
 	gint64	speed_limit;
 
-	if (parameter != UG_DATA_INT64)
+	if (parameter != UG_TYPE_INT64)
 		return UG_RESULT_UNSUPPORT;
 
 	speed_limit = *(gint64*)data;
@@ -228,7 +228,7 @@ static UgResult	ug_plugin_curl_get (UgPluginCurl* plugin, guint parameter, gpoin
 	gdouble		total;
 	gdouble		speed;
 
-	if (parameter != UG_DATA_INSTANCE)
+	if (parameter != UG_TYPE_INSTANCE)
 		return UG_RESULT_UNSUPPORT;
 	if (data == NULL || ((UgData*)data)->iface != UG_PROGRESS_I)
 		return UG_RESULT_UNSUPPORT;
@@ -287,12 +287,15 @@ static gpointer	ug_plugin_curl_thread (UgPluginCurl* plugin)
 	// setup scheme and it's related data (user & password)
 	ug_plugin_curl_set_scheme (plugin, curl);
 	// check filename
-	if (common->file == NULL) {
+	if (common->file)
+		plugin->keep_filename = TRUE;
+	else {
 		common->file = ug_uri_get_filename (common->url);
 		if (common->file == NULL)
 			common->file = g_strdup ("index.htm");
 		ug_plugin_post ((UgPlugin*) plugin,
 				ug_message_new_data (UG_MESSAGE_DATA_FILE_CHANGED, common->file));
+		plugin->keep_filename = FALSE;
 	}
 	if (common->max_download_speed) {
 		curl_easy_setopt (curl, CURLOPT_MAX_SEND_SPEED_LARGE,
@@ -413,6 +416,7 @@ static gpointer	ug_plugin_curl_thread (UgPluginCurl* plugin)
 
 	// Others -----------------------------------------------------------------
 	curl_easy_setopt (curl, CURLOPT_FILETIME, 1);	// use with CURLINFO_FILETIME
+	curl_easy_setopt (curl, CURLOPT_SSL_VERIFYPEER, FALSE);	// disable peer SSL certificate verification
 	curl_easy_setopt (curl, CURLOPT_ERRORBUFFER, plugin->curl_error_string);
 	curl_easy_setopt (curl, CURLOPT_LOW_SPEED_LIMIT, 100);
 	curl_easy_setopt (curl, CURLOPT_LOW_SPEED_TIME, 60);
@@ -463,7 +467,9 @@ static gpointer	ug_plugin_curl_thread (UgPluginCurl* plugin)
 			goto exit;
 
 		// can resume (retry)
+		case CURLE_RECV_ERROR:
 		case CURLE_PARTIAL_FILE:
+		case CURLE_OPERATION_TIMEDOUT:
 			// update status
 			plugin->resumable = TRUE;
 			// send message
@@ -486,9 +492,8 @@ static gpointer	ug_plugin_curl_thread (UgPluginCurl* plugin)
 			break;
 
 		// retry
+		case CURLE_SEND_ERROR:
 		case CURLE_GOT_NOTHING:
-		case CURLE_RECV_ERROR:
-		case CURLE_OPERATION_TIMEDOUT:
 		case CURLE_BAD_CONTENT_ENCODING:
 			ug_plugin_post ((UgPlugin*) plugin,
 					ug_message_new_error (UG_MESSAGE_ERROR_CUSTOM, plugin->curl_error_string));
@@ -567,9 +572,11 @@ exit:
 #else
 		ug_plugin_rename_file ((UgPlugin*) plugin, plugin->path_tmp, plugin->path);
 #endif
-		curl_easy_getinfo (curl, CURLINFO_FILETIME, &curl_time);
-		if (curl_time != -1)
-			ug_modify_file_time (plugin->path, (time_t) curl_time);
+		if (common->retrieve_timestamp) {
+			curl_easy_getinfo (curl, CURLINFO_FILETIME, &curl_time);
+			if (curl_time != -1)
+				ug_modify_file_time (plugin->path, (time_t) curl_time);
+		}
 		ug_plugin_post ((UgPlugin*) plugin,
 				ug_message_new_info (UG_MESSAGE_INFO_COMPLETE, NULL));
 		ug_plugin_post ((UgPlugin*) plugin,
@@ -779,8 +786,8 @@ static size_t	ug_plugin_curl_header_http (char *buffer, size_t size, size_t nmem
 				ug_message_new_data (UG_MESSAGE_DATA_HTTP_LOCATION, common->url));
 		// setup scheme and it's related data (user & password)
 		ug_plugin_curl_set_scheme (plugin, plugin->curl);
-//		if (common->keeping.file == FALSE)
-		file = ug_uri_get_filename (common->url);
+		if (plugin->keep_filename == FALSE)
+			file = ug_uri_get_filename (common->url);
 	}
 	// handle HTTP header "Content-Location:"
 	else if (g_ascii_strncasecmp (buffer, "Content-Location: ", 18) == 0) {
@@ -789,8 +796,8 @@ static size_t	ug_plugin_curl_header_http (char *buffer, size_t size, size_t nmem
 		temp = g_strndup (buffer, strcspn (buffer, "\r\n"));
 		ug_plugin_post ((UgPlugin*) plugin,
 				ug_message_new_data (UG_MESSAGE_DATA_HTTP_CONTENT_LOCATION, temp));
-//		if (common->keeping.file == FALSE)
-		file = ug_uri_get_filename (temp);
+		if (plugin->keep_filename == FALSE)
+			file = ug_uri_get_filename (temp);
 		g_free (temp);
 	}
 	// handle HTTP header "Content-Disposition:"
@@ -801,18 +808,20 @@ static size_t	ug_plugin_curl_header_http (char *buffer, size_t size, size_t nmem
 		ug_plugin_post ((UgPlugin*) plugin,
 				ug_message_new_data (UG_MESSAGE_DATA_HTTP_CONTENT_DISPOSITION, buffer));
 		// grab filename
-		file = strstr (buffer, "filename=");
-		if (file) {
-			file += 9;	// value of "filename="
-			if (file[0] == '\"') {
-				file += 1;
-				temp = g_strndup (file, strcspn (file, "\""));
+		if (plugin->keep_filename == FALSE) {
+			file = strstr (buffer, "filename=");
+			if (file) {
+				file += 9;	// value of "filename="
+				if (file[0] == '\"') {
+					file += 1;
+					temp = g_strndup (file, strcspn (file, "\""));
+				}
+				else
+					temp = g_strndup (file, strcspn (file, ";"));
+//				if (common->keeping.file == FALSE)
+				file = ug_uri_get_filename (temp);
+				g_free (temp);
 			}
-			else
-				temp = g_strndup (file, strcspn (file, ";"));
-//			if (common->keeping.file == FALSE)
-			file = ug_uri_get_filename (temp);
-			g_free (temp);
 		}
 		g_free (buffer);
 	}
