@@ -1,6 +1,6 @@
 /*
  *
- *   Copyright (C) 2005-2012 by C.H. Huang
+ *   Copyright (C) 2005-2013 by C.H. Huang
  *   plushuang.tw@gmail.com
  *
  *  This library is free software; you can redistribute it and/or
@@ -107,6 +107,10 @@ void	ug_xmlrpc_finalize (UgXmlrpc* xmlrpc)
 
 void	ug_xmlrpc_use_client (UgXmlrpc* xmlrpc, const gchar* uri, const gchar* user_agent)
 {
+	if (xmlrpc->uri)
+		g_free (xmlrpc->uri);
+	if (xmlrpc->user_agent)
+		g_free (xmlrpc->user_agent);
 	xmlrpc->uri = g_strdup (uri);
 	xmlrpc->user_agent = g_strdup (user_agent ? user_agent : "uGet/1.7");
 
@@ -409,6 +413,7 @@ static void	ug_xmltag_parse_response (UgXmltag* xmltag, UgXmlrpc* xmlrpc)
 
 	default:
 		ug_xmltag_push (xmltag, ug_xmltag_parse_unknown, NULL);
+		break;
 	}
 }
 
